/*------------------------------------------------------------------------------
 bin/djv_info/app_cmdln.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "app.h"
#include "etc.h"
#include <djv_image/io.h>

namespace djv_info {

void App::cmdln(Str & in) throw (Error)
{
  djv_image::app::App::cmdln(in);
  if (exit() != djv_base::app::EXIT_DEFAULT) return;
  
  if (in.empty())
  {
    exit(djv_base::app::EXIT_HELP);
    return;
  }

  Str arg;
try
{
  while (! in.empty())
  {
    in >> arg;
    
    // Options.
    
    if ("-verbose" == arg || "-v" == arg) _verbose = true;
    else if ("-seq" == arg || "-q" == arg) in >> _seq;
    else if ("-recurse" == arg || "-r" == arg) _recurse = true;
    else if ("-columns" == arg || "-c" == arg) in >> _columns;
    
    // Arguments.
    
    else _input += arg;
  }
}
catch (Str) { throw Error(arg); }

  if (! _input.size()) throw Error(error_cmdln_input);
}

Str App::cmdln_help() const
{
  return str::Format(label_cmdln_help).
    arg(file::label_seq_compress(), ", ").
    arg(str::label(_seq)).
    arg(djv_image::app::App::cmdln_help());
}

const Str label_cmdln_help =
"\n"
"djv_info\n"
"--------\n"
"\n"
"Command line utility to display image information.\n"
"\n"
"Output\n"
"------\n"
"\n"
"Format:\n"
"\n"
"(name) (width)x(height):(aspect) (format) (type) (duration):(speed)\n"
"  (layer): (name) (width)x(height):(aspect) (format) (type)\n"
"\n"
"Example:\n"
"\n"
"yesterdayview.mov                         640x424:1.51 RGB U8 00:02:00:01@12\n"
"dlad.dpx                               2048x1556:1.32 RGB U10 00:00:00:01@24\n"
"render0001-1000.exr                       720x480:1.5 RGB F16 00:00:41:16@24\n"
"  1: Ambient                                             720x480:1.5 RGB F16\n"
"  2: Diffuse                                             720x480:1.5 RGB F16\n"
"  3: Specular                                            720x480:1.5 RGB F16\n"
"  4: Z                                                     720x480:1.5 L F32\n"
"\n"
"Usage\n"
"-----\n"
"\n"
"* djv_info (image|directory)... [option]...\n"
"\n"
"Options\n"
"-------\n"
"\n"
"* -verbose, -v\n"
"  Verbose information.\n"
"\n"
"* -seq, -q (value)\n"
"  File sequencing: %%. Value: %%.\n"
"\n"
"* -recurse, -r\n"
"  Descend into sub-directories.\n"
"\n"
"* -columns, -c (value)\n"
"  Number of columns for formatting. A value of zero disables formatting.\n"
"\n"
"General Options\n"
"---------------\n"
"\n"
"%%"
"\n"
"Examples\n"
"--------\n"
"\n"
"Display image information:\n"
"> djv_info image.sgi image2.sgi\n"
"\n"
"Display image sequence information:\n"
"> djv_info image.1-100.sgi\n"
"\n"
"Display Information about all images within a directory:\n"
"> djv_info ~/pics\n"
"\n"
"Display video lookup table information:\n"
"> djv_info 0.vlut\n"
"\n";

}

