/*------------------------------------------------------------------------------
 bin/djv_view/image_sample.h
 
 Image Sample
 ------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_IMAGE_SAMPLE_H
#define DJV_VIEW_IMAGE_SAMPLE_H

#include <djv_ui/dialog.h>

namespace djv_view {
namespace view { class Widget; }
namespace image_sample {
using namespace djv_base;

//------------------------------------------------------------------------------
// Options

enum SIZE
{
  SIZE_1,
  SIZE_3,
  SIZE_5,
  SIZE_7,
  SIZE_9,
  SIZE_51,
  SIZE_101
};

//------------------------------------------------------------------------------
// Dialog

class Dialog : public djv_ui::dialog::Base
{
public:
  
  Dialog();
  ~Dialog();
  void del();

  // Value.
  
  void set(const view::Widget *, const Vec2i &);
  void pick(const view::Widget *, const Vec2i &);
  void update(const view::Widget *);
  void del(const view::Widget *);
  
  // Widget.
  
  void show();
  
  // Layout.
  
  void update() { Base::update(); }

private:

  DJV_CALLBACK(Dialog, widget_callback, const djv_image::Color &);
  DJV_CALLBACK(Dialog, size_callback, int);
  DJV_CALLBACK(Dialog, profile_callback, bool);
  DJV_CALLBACK(Dialog, lock_callback, bool);
  DJV_CALLBACK(Dialog, close_callback, bool);

  void widget_update();
  
  const view::Widget * _view;
  Vec2i _pick;
  djv_image::Color _value;
  SIZE _size;
  bool _profile, _lock;
  
  struct _Dialog;
  std::auto_ptr<_Dialog> _p;
};

//------------------------------------------------------------------------------
// Functions

int size(SIZE);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, SIZE &) throw (Str);
Str & operator << (Str &, SIZE);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_size();

extern const Str
  dialog_title,
  dialog_size,
  dialog_profile,
  dialog_lock;

extern const Str
  tooltip_size,
  tooltip_profile,
  tooltip_lock;

//------------------------------------------------------------------------------
// Global

Dialog * dialog();

}
}

#endif

