/*------------------------------------------------------------------------------
 bin/djv_view/playback.h
 
 Playback
 --------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_PLAYBACK_H
#define DJV_VIEW_PLAYBACK_H

#include "widget.h"
#include <djv_ui/menu.h>
#include <djv_image/time.h>

namespace djv_view {
namespace playback {
using namespace djv_base;

//------------------------------------------------------------------------------
// Options

enum PLAYBACK
{
  STOP,
  FORWARD,
  REVERSE
};

enum TIMER
{
  TIMER_SLEEP,
  TIMER_TIMEOUT
};

//------------------------------------------------------------------------------
// Playabck Group

class Group : public djv_ui::Callback_Base
{
public:
  
  Group(
    djv_ui::menu::Widget *,
    djv_ui::group::Frame * playbar,
    Group * copy = 0
  );
  ~Group();

  // Value.
  
  void playback(PLAYBACK);
  
  void time(const djv_image::Time &);
  const djv_image::Time & time() const;

  void frame(int, bool wrap = true);
  int frame() const;
  int in() const;
  int out() const;

  void speed(const djv_image::time::Speed &);
  const djv_image::time::Speed & speed() const;
  double speed_real() const;
  bool dropped_frames() const;

  void cache(const List<int> &);
  
  // Signals.
  
  djv_ui::Signal<bool> image_sig, overlay_sig, update_sig;
  
  // Widget.
  
  void menu_update(djv_ui::menu::Group *);
  void playbar_update();
  void idle();

private:
  
  DJV_CALLBACK(Group, playback_callback, PLAYBACK);
  DJV_CALLBACK(Group, stop_callback, bool);
  DJV_CALLBACK(Group, forward_callback, bool);
  DJV_CALLBACK(Group, reverse_callback, bool);
  DJV_CALLBACK(Group, playback_shuttle_callback, bool);
  DJV_CALLBACK(Group, playback_shuttle_value_callback, int);
  DJV_CALLBACK(Group, frame_callback, int);
  DJV_CALLBACK(Group, frame_slider_callback, int);
  DJV_CALLBACK(Group, frame_shuttle_callback, bool);
  DJV_CALLBACK(Group, frame_shuttle_value_callback, int);
  DJV_CALLBACK(Group, start_callback, bool);
  DJV_CALLBACK(Group, start_abs_callback, bool);
  DJV_CALLBACK(Group, end_callback, bool);
  DJV_CALLBACK(Group, end_abs_callback, bool);
  DJV_CALLBACK(Group, prev_callback, bool);
  DJV_CALLBACK(Group, next_callback, bool);
  DJV_CALLBACK(Group, in_callback, int);
  DJV_CALLBACK(Group, in_mark_callback, bool);
  DJV_CALLBACK(Group, in_reset_callback, bool);
  DJV_CALLBACK(Group, out_callback, int);
  DJV_CALLBACK(Group, out_mark_callback, bool);
  DJV_CALLBACK(Group, out_reset_callback, bool);
  DJV_CALLBACK(Group, speed_callback, const djv_image::time::Speed &);
  DJV_CALLBACK(Group, every_frame_callback, bool);
  DJV_FL_WIDGET_CALLBACK(Group, _stop_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _forward_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _reverse_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _start_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _start_abs_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _end_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _end_abs_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _prev_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _prev_x10_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _prev_x100_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _next_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _next_x10_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _next_x100_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _in_goto_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _in_mark_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _in_reset_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _out_goto_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _out_mark_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _out_reset_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _every_frame_callback);
  
  void playback_update();
  void time_update();
  void frame_update();
  void speed_update();
  
  PLAYBACK _playback;
  djv_image::Time _time;
  int _frame, _frame_tmp;
  int _in, _out;
  bool _every_frame;
  bool _shuttle;
  double _shuttle_speed;
  djv_image::time::Speed _speed;
  double _speed_real;
  bool _dropped_frames, _dropped_frames_tmp;
  bool _idle_init;
  djv_base::time::Timer _idle_timer, _speed_timer;
  uint64_t _idle_frame, _speed_counter;
  
  struct _Group;
  std::auto_ptr<_Group> _p;
};

//------------------------------------------------------------------------------
// Preferences

class Prefs : public djv_ui::Callback_Base
{
public:

  Prefs();
  ~Prefs();

  // Options.
  
  void start(bool);
  bool start() const;

  void every_frame(bool);
  bool every_frame() const;
  djv_ui::Signal<bool> every_frame_sig;
  
private:

  bool _start, _every_frame;
};

//------------------------------------------------------------------------------
// Preferences Widget

class Prefs_Widget : public djv_ui::widget::Widget
{
public:

  Prefs_Widget();

private:

  DJV_CALLBACK(Prefs_Widget, start_callback, bool);
  DJV_CALLBACK(Prefs_Widget, every_frame_callback, bool);
};

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, PLAYBACK &) throw (Str);
Str & operator >> (Str &, TIMER &) throw (Str);
Str & operator << (Str &, PLAYBACK);
Str & operator << (Str &, TIMER);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_playback();
const List<Str> & label_timer();

extern const Str
  menu_title,
  menu_stop,
  menu_forward,
  menu_reverse,
  menu_start,
  menu_start_abs,
  menu_end,
  menu_end_abs,
  menu_prev,
  menu_prev_x10,
  menu_prev_x100,
  menu_next,
  menu_next_x10,
  menu_next_x100,
  menu_in_mark,
  menu_in_reset,
  menu_out_mark,
  menu_out_reset,
  menu_every_frame;

extern const Str
  tooltip_title,
  tooltip_stop,
  tooltip_forward,
  tooltip_reverse,
  tooltip_playback_shuttle,
  tooltip_frame,
  tooltip_frame_slider,
  tooltip_frame_shuttle,
  tooltip_start,
  tooltip_start_button,
  tooltip_end,
  tooltip_end_button,
  tooltip_prev,
  tooltip_next,
  tooltip_duration,
  tooltip_in_mark,
  tooltip_in_reset,
  tooltip_out_mark,
  tooltip_out_reset,
  tooltip_speed,
  tooltip_speed_real,
  tooltip_every_frame;

extern const Str
  prefs_title,
  prefs_group,
  prefs_start,
  prefs_every_frame;

//------------------------------------------------------------------------------
// Global

class Global : public djv_ui::Callback_Base
{
public:

  Global();
  ~Global();

  void timer(TIMER);
  void timer_resolution(double);
  void timer_add(Group *);
  void timer_del(Group *);
  TIMER timer() const;
  double timer_resolution() const;

private:

  DJV_FL_CALLBACK(Global, timer_callback);

  TIMER _timer;
  List<playback::Group *> _timer_list;
  double _timer_resolution;
  djv_base::time::Timer _timer_time;
};

Prefs * prefs();
Global * global();

}
}

#endif

