/*------------------------------------------------------------------------------
 bin/djv_view/playback_etc.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "playback.h"

namespace djv_view {
namespace playback {

DJV_STR_OPERATOR_SET(PLAYBACK, label_playback())
DJV_STR_OPERATOR_GET(PLAYBACK, label_playback())
DJV_STR_OPERATOR_SET(TIMER, label_timer())
DJV_STR_OPERATOR_GET(TIMER, label_timer())

const List<Str> & label_playback()
{
  static const List<Str> data = List<Str>() <<
    "Stop" <<
    "Forward" <<
    "Reverse";
  return data;
}

const List<Str> & label_timer()
{
  static const List<Str> data = List<Str>() <<
    "Sleep" <<
    "Timeout";
  return data;
}

const Str
  menu_title = "&Playback",
  menu_stop = "Stop",
  menu_forward = "Forward",
  menu_reverse = "Reverse",
  menu_start = "Start",
  menu_start_abs = "Start Absolute",
  menu_end = "End",
  menu_end_abs = "End Absolute",
  menu_prev = "Previous",
  menu_prev_x10 = "Previous X10",
  menu_prev_x100 = "Previous X100",
  menu_next = "Next",
  menu_next_x10 = "Next X10",
  menu_next_x100 = "Next X100",
  menu_in_mark = "In Mark",
  menu_in_reset = "In Reset",
  menu_out_mark = "Out Mark",
  menu_out_reset = "Out Reset",
  menu_every_frame = "Every Frame";

const Str
  tooltip_stop = "Stop playback.\nKeyboard shortcut: %%",
  tooltip_forward = "Forward playback.\nKeyboard shortcut: %%",
  tooltip_reverse = "Reverse playback.\nKeyboard shortcut: %%",
  tooltip_playback_shuttle = "Playback shuttle. Click and drag to start "
  "playback. The playback speed is determined by how far you drag. Used for "
  "fast-forwarding.",
  tooltip_frame = "Current frame.",
  tooltip_frame_slider = "Frame slider.",
  tooltip_frame_shuttle = "Frame shuttle. Click and drag to change the "
  "current frame.",
  tooltip_start = "Start frame / in point.",
  tooltip_start_button = "Start frame.\nKeyboard shortcut: %%",
  tooltip_end = "End frame / out point.",
  tooltip_end_button = "End frame.\nKeyboard shortcut: %%",
  tooltip_prev = "Previous frame.\nKeyboard shortcut: %%",
  tooltip_next = "Next frame.\nKeyboard shortcut: %%",
  tooltip_in_mark = "Mark in point.\nKeyboard shortcut: %%",
  tooltip_in_reset = "Reset in point.\nKeyboard shortcut: %%",
  tooltip_out_mark = "Mark out point.\nKeyboard shortcut: %%",
  tooltip_out_reset = "Reset out point.\nKeyboard shortcut: %%",
  tooltip_duration = "Playback duration.",
  tooltip_speed = "Playback speed.",
  tooltip_speed_real = "Real playback speed.",
  tooltip_every_frame = "Playback every frame.";

const Str
  prefs_title = "Playback",
  prefs_group = "Options",
  prefs_start = "Start playback when opening files.",
  prefs_every_frame = "Play every frame.";

Global::Global() :
  _timer(TIMER_SLEEP),
  _timer_resolution(0.001)
{}

Global::~Global()
{
  Fl::remove_idle(timer_callback, this);
  Fl::remove_timeout(timer_callback, this);
}

void Global::timer(TIMER in)
{
  _timer = in;
}

TIMER Global::timer() const { return _timer; }

void Global::timer_resolution(double in)
{
  _timer_resolution = in;
}

double Global::timer_resolution() const { return _timer_resolution; }

void Global::timer_add(Group * in)
{
  DJV_DBG_OBJ("Global::timer_add");
  
  _timer_list += in;
  
  if (1 == _timer_list.size())
    switch (_timer)
    {
      case TIMER_SLEEP:
        _timer_time.start();
        Fl::add_idle(timer_callback, this);
        break;
      
      case TIMER_TIMEOUT:
        Fl::add_timeout(_timer_resolution, timer_callback, this);
        break;
    }
}

void Global::timer_del(Group * in)
{
  DJV_DBG_OBJ("Global::timer_del");

  List<playback::Group *>::iterator i = std::find(
    _timer_list.begin(), _timer_list.end(), in
  );
  if (i != _timer_list.end()) _timer_list.erase(i);
  
  if (! _timer_list.size())
  {
    Fl::remove_idle(timer_callback, this);
    Fl::remove_timeout(timer_callback, this);
  }
}

void Global::timer_callback()
{
  DJV_DBG_OBJ("Global::timer_callback");
  DJV_DBG_PRN("size = " << int(_timer_list.size()));
  
  for (uint i = 0; i < _timer_list.size(); ++i)
    _timer_list[i]->idle();
  
  switch (_timer)
  {
    case TIMER_SLEEP:
      while (_timer_time.seconds() < _timer_resolution)
      {
        djv_base::time::sleep(0.0);
        _timer_time.check();
      }
      _timer_time.start();
      break;
      
    case TIMER_TIMEOUT:
      Fl::repeat_timeout(_timer_resolution, timer_callback, this);
      break;
  }
}

}
}
