/*------------------------------------------------------------------------------
 include/djv_base/box3_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_base {
namespace box {

template<typename T>
const int Box<T, 3>::dimension = 3;

#define _BOX3_INIT \
  \
  x(this->position.e[0]), \
  y(this->position.e[1]), \
  z(this->position.e[2]), \
  w(this->size.e[0]), \
  h(this->size.e[1]), \
  d(this->size.e[2])

template<typename T>
inline Box<T, 3>::Box() :
  _BOX3_INIT
{}

template<typename T>
inline Box<T, 3>::Box(const Box<T, 3> & in) :
  position(in.position),
  size(in.size),
  _BOX3_INIT
{}

template<typename T>
inline Box<T, 3>::Box(const Vec<T, 3> & position, const Vec<T, 3> & size) :
  position(position),
  size(size),
  _BOX3_INIT
{}

template<typename T>
inline Box<T, 3>::Box(const Vec<T, 3> & size) :
  size(size),
  _BOX3_INIT
{}

template<typename T>
inline Box<T, 3>::Box(T _x, T _y, T _z, T _w, T _h, T _d) :
  _BOX3_INIT
{
  x = _x;
  y = _y;
  z = _z;
  w = _w;
  h = _h;
  d = _d;
}

template<typename T>
inline Box<T, 3>::Box(T _w, T _h, T _d) :
  _BOX3_INIT
{
  w = _w;
  h = _h;
  d = _d;
}

template<typename T>
inline Box<T, 3> & Box<T, 3>::operator = (const Box<T, 3> & in)
{
  if (&in != this) set(in);
  return *this;
}

template<typename T>
inline void Box<T, 3>::set(const Box<T, 3> & in)
{
  position = in.position;
  size = in.size;
}

template<typename T>
inline void Box<T, 3>::position1(const Vec<T, 3> & in)
{
  size = in - position;
}

template<typename T>
inline Vec<T, 3> Box<T, 3>::position1() const
{
  return position + size;
}

template<typename T>
inline void Box<T, 3>::zero()
{
  position.zero();
  size.zero();
}

// XXX Specialization for integer types.

template<>
inline void Box<int, 3>::position1(const Vec<int, 3> & in)
{
  size = in - position + 1;
}

template<>
inline Vec<int, 3> Box<int, 3>::position1() const
{
  return position + size - 1;
}

template<typename T>
inline Box<T, 3>::operator Box<int, 3> () const
{
  return Box<int, 3>(
    int(x), int(y), int(z),
    int(w), int(h), int(d));
}

template<typename T>
inline Box<T, 3>::operator Box<double, 3> () const
{
  return Box<double, 3>(
    double(x), double(y), double(z),
    double(w), double(h), double(d));
}

}
}
