/*------------------------------------------------------------------------------
 include/djv_base/list.h
 
 Lists
 -----
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_LIST_H
#define DJV_LIST_H

#include <djv_base/type.h>
#include <vector>

namespace djv_base {
namespace list {

//------------------------------------------------------------------------------
// List

template<typename T>
class List : public std::vector<T>
{
public:

  inline List();
  inline List(const List<T> &);
  inline List(const List<T> &, uint position, uint size);
  inline List(const T &, uint size);
  inline List(uint size);
  
  // Push/pop.

  inline void push_front(const T &);
  
  inline T pop_front();
  inline T pop_back();
  
  // Append.

  inline void append(const T &);
  inline void append(const List<T> &);

  inline List<T> & operator += (const T &);
  inline List<T> & operator += (const List<T> &);
  
  inline List<T> & operator << (const T &);
  inline List<T> & operator << (const List<T> &);
};

//------------------------------------------------------------------------------
// Functions

template<typename T>
inline int find(const T &, const List<T> &);

}
}

#include <djv_base/list_.h>

#endif

