/*------------------------------------------------------------------------------
 include/djv_base/mem.h
 
 Memory
 ------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_MEM_H
#define DJV_MEM_H

#include <djv_base/list.h>

namespace djv_base {
namespace mem {

//------------------------------------------------------------------------------
// Constants

const uint64_t kilobyte = 1024;
const uint64_t megabyte = kilobyte * 1024;
const uint64_t gigabyte = megabyte * 1024;
const uint64_t terabyte = gigabyte * 1024;

const size_t align = 4096;

//------------------------------------------------------------------------------
// Endian

enum ENDIAN
{
  MSB,
  LSB
};

//------------------------------------------------------------------------------
// Memory Buffer

template<typename T>
class Buffer
{
public:

  inline Buffer();
  inline Buffer(const Buffer &);
  inline Buffer(size_t);
  inline ~Buffer();
  inline Buffer & operator = (const Buffer &);
  
  // Size.
  
  inline void size(size_t);
  inline size_t size() const;
  
  // Data.
  
  inline T * data();
  inline T * operator () ();
  inline const T * data() const;
  inline const T * operator () () const;

  // Functions.
  
  inline void zero();

private:

  inline void del();

  T * _data;
  size_t _size;
};

//------------------------------------------------------------------------------
// Functions

// Allocate/deallocate.

void * get(size_t);
void del(void *);

// Copy.

void copy(const void *, void *, size_t size);

// Set.

void set(uint8_t, void *, size_t size);

// Zero.

void zero(void *, size_t size);

// Endian.

inline ENDIAN endian();
inline ENDIAN endian_opposite(ENDIAN);

//------------------------------------------------------------------------------
// Comparison

int compare(const void *, const void *, size_t size);

//------------------------------------------------------------------------------
// Conversion

inline void endian(void *, size_t size, size_t word_size);
inline void endian(const void *, void *, size_t size, size_t word_size);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_endian();

}
}

#include <djv_base/mem_.h>

#endif

