/*------------------------------------------------------------------------------
 include/djv_base/mtx_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_base {
namespace mtx {

template<typename T, int D>
const int Mtx<T, D>::dimension = D;
template<typename T, int D>
const int Mtx<T, D>::dimension_x2 = D * D;

template<typename T, int D>
inline Mtx<T, D>::Mtx()
{
  identity();
}

template<typename T, int D>
inline Mtx<T, D>::Mtx(const Mtx<T, D> & in)
{
  set(in);
}

template<typename T, int D>
inline Mtx<T, D>::Mtx(const T * in)
{
  set(in);
}

template<typename T, int D>
inline void Mtx<T, D>::set(const Mtx<T, D> & in)
{
  for (int i = 0; i < dimension_x2; ++i)
    e[i] = in.e[i];
}

template<typename T, int D>
inline void Mtx<T, D>::set(const T * in)
{
  for (int i = 0; i < dimension_x2; ++i)
    e[i] = in[i];
}

template<typename T, int D>
inline void Mtx<T, D>::identity()
{
  for (int i = 0; i < D; ++i)
    for (int j = 0; j < D; ++j)
      e[i * D + j] = (i == j) ? 1.0 : 0.0;
}

template<typename T, int D>
inline void Mtx<T, D>::zero()
{
  for (int i = 0; i < dimension_x2; ++i)
    e[i] = 0.0;
}

}
}

