/*------------------------------------------------------------------------------
 include/djv_base/vec_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_base {
namespace vec {

template<typename T, int D>
const int Vec<T, D>::dimension = D;

template<typename T, int D>
inline Vec<T, D>::Vec()
{
  zero();
}

template<typename T, int D>
inline Vec<T, D>::Vec(const Vec<T, D> & in)
{
  set(in);
}

template<typename T, int D>
inline Vec<T, D>::Vec(T in)
{
  set(in);
}

template<typename T, int D>
inline Vec<T, D> & Vec<T, D>::operator = (const Vec<T, D> & in)
{
  if (&in != this) set(in);
  return *this;
}

template<typename T, int D>
inline void Vec<T, D>::set(const Vec<T, D> & in)
{
  for (int i = 0; i < dimension; ++i)
    Vec<T, D>::e[i] = in.e[i];
}

template<typename T, int D>
inline void Vec<T, D>::set(T in)
{
  for (int i = 0; i < dimension; ++i)
    e[i] = in;
}

template<typename T, int D>
inline void Vec<T, D>::zero()
{
  for (int i = 0; i < dimension; ++i)
    e[i] = 0;
}

#define _VEC_OP(IN) \
  \
  template<typename T, int D> \
  inline Vec<T, D> & Vec<T, D>::operator IN (const Vec<T, D> & in) \
  { \
    for (int i = 0; i < dimension; ++i) \
      Vec<T, D>::e[i] IN in.e[i]; \
    return *this; \
  }

_VEC_OP(+=)
_VEC_OP(-=)
_VEC_OP(*=)
_VEC_OP(/=)

#define _VEC_OP2(IN) \
  \
  template<typename T, int D> \
  inline Vec<T, D> & Vec<T, D>::operator IN (T in) \
  { \
    for (int i = 0; i < dimension; ++i) \
      Vec<T, D>::e[i] IN in; \
    return *this; \
  }

_VEC_OP2(+=)
_VEC_OP2(-=)
_VEC_OP2(*=)
_VEC_OP2(/=)

}
}

