/*------------------------------------------------------------------------------
 include/djv_image/color.h
 
 Color
 -----
  
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_COLOR_H
#define DJV_IMAGE_COLOR_H

#include <djv_image/pixel.h>

namespace djv_image {
namespace color {

//------------------------------------------------------------------------------
// Color

class Color
{
public:

  Color();
  Color(const Color &);
  Color(PIXEL);
  Color(pixel::F32_T l);
  Color(pixel::F32_T l, pixel::F32_T a);
  Color(pixel::F32_T r, pixel::F32_T g, pixel::F32_T b);
  Color(pixel::F32_T r, pixel::F32_T g, pixel::F32_T b, pixel::F32_T a);
  Color & operator = (const Color &);

  // Value.
  
  void set(PIXEL);
  inline PIXEL pixel() const;
  inline int channels() const;
  
  // Components.
  
  inline void set_u8(pixel::U8_T, int);
  inline void set_u10(pixel::U10_T, int);
  inline void set_u16(pixel::U16_T, int);
  inline void set_f16(pixel::F16_T, int);
  inline void set_f32(pixel::F32_T, int);
  
  inline pixel::U8_T get_u8(int) const;
  inline pixel::U10_T get_u10(int) const;
  inline pixel::U16_T get_u16(int) const;
  inline pixel::F16_T get_f16(int) const;
  inline pixel::F32_T get_f32(int) const;

  // Functions.
  
  void zero();
  
  // Data access.

  inline uint8_t * data();
  inline const uint8_t * data() const;

private:

  void init();

  PIXEL _pixel;
  int _channels;
  uint8_t _data [pixel::channels_max * pixel::bytes_max];
};

//------------------------------------------------------------------------------
// Comparison

bool operator == (const Color &, const Color &);
bool operator != (const Color &, const Color &);

//------------------------------------------------------------------------------
// Conversion

void convert(const Color &, Color *);

inline void rgb_to_hsv(const double [3], double [3]);
inline void hsv_to_rgb(const double [3], double [3]);

Str & operator >> (Str &, Color &) throw (Str);
Str & operator << (Str &, const Color &);

//------------------------------------------------------------------------------
// Debugging

Dbg & operator << (Dbg &, const Color &);

}

//------------------------------------------------------------------------------
// Declarations

using color::Color;

}

#include <djv_image/color_.h>

#endif

