/*------------------------------------------------------------------------------
 include/djv_image/io_lut.h
 
 Image I/O - LUT
 ---------------
 
 Lookup table file format.
 
 Lookup tables provide a way to store basic color transformations.

 Support:
 
 * File types: Inferno, Kodak.
 * Image data:
   * L, LA, RGB, RGBA, 8-bit, 16-bit.
   * RGB, 10-bit.

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_LUT_H
#define DJV_IMAGE_IO_LUT_H

#include <djv_image/io.h>

namespace djv_image {
namespace io_lut {

//------------------------------------------------------------------------------
// Information

static const Str name = "LUT";

static const List<Str> extension_list = List<Str>() <<
  ".lut" <<
  ".1dl";

//------------------------------------------------------------------------------
// Options

enum SPEC
{
  SPEC_AUTO,
  SPEC_INFERNO,
  SPEC_KODAK
};

enum TYPE
{
  TYPE_AUTO,
  TYPE_U8,
  TYPE_U10,
  TYPE_U16
};
  
enum OPTION
{
  LOAD_SPEC,
  LOAD_TYPE
};

struct Load_Option
{
  Load_Option() :
    spec(SPEC_AUTO),
    type(TYPE_AUTO)
  {}

  SPEC spec;
  TYPE type;
};

enum SAVE_OPTION
{
  SAVE_SPEC
};

struct Save_Option
{
  Save_Option() :
    spec(SPEC_AUTO)
  {}

  SPEC spec;
};

//------------------------------------------------------------------------------
// Load Plugin

class Load : public io::Load
{
public:
  
  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, Info *, Tag *, Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _open(const File &, Info *) throw (Error);

  Load_Option _option;
  File _file;
  djv_base::Io _io;
  SPEC _spec;
  Image _image;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public io::Save
{
public:
  
  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, const Info &, const Tag &, const Time &)
    throw (Error);
  void set(const Image &, int frame) throw (Error);

private:

  void _open(const File &) throw (Error);
  
  Save_Option _option;
  File _file;
  djv_base::Io _io;
  SPEC _spec;
  Info _info;
  Image _image;
};

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, SPEC &) throw (Str);
Str & operator >> (Str &, TYPE &) throw (Str);
Str & operator << (Str &, SPEC);
Str & operator << (Str &, TYPE);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_spec();
const List<Str> & label_type();
const List<Str> & label_load_option();
const List<Str> & label_save_option();

}
}

#endif

