/*------------------------------------------------------------------------------
 include/djv_ui/button.h

 Button Widgets
 --------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_BUTTON_H
#define DJV_UI_BUTTON_H

#include <djv_ui/widget.h>
#include <FL/Fl_Image.H>

namespace djv_ui {
namespace button {

//------------------------------------------------------------------------------
// Push Button

class Push : public widget::Widget
{
public:

  Push(const Str & in);
  virtual ~Push();
  
  // Value.

  Signal<bool> sig;
  
  // Label.

  void label(const Str &);
  const Str & label() const;
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::button::Push"; }

private:
  
  DJV_FL_WIDGET_CALLBACK(Push, widget_callback);
  
  Str _label;
  
  struct _Push;
  std::auto_ptr<_Push> _p;
};

//------------------------------------------------------------------------------
// Check Button

class Check : public widget::Widget
{
public:

  Check(const Str & = Str());
  virtual ~Check();
  
  // Value.

  void set(bool);
  bool get() const;
  Signal<bool> sig;
  
  // Label.

  void label(const Str &);
  const Str & label() const;
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::button::Check"; }
  
private:
  
  DJV_FL_WIDGET_CALLBACK(Check, widget_callback);

  bool _value;
  Str _label, _label_tmp;
  
  struct _Check;
  std::auto_ptr<_Check> _p;
};

//------------------------------------------------------------------------------
// Radio Button

class Radio : public widget::Widget
{
public:

  Radio(const Str & = Str());
  virtual ~Radio();
  
  // Value.

  void set(bool = true);
  bool get() const;
  Signal<bool> sig;
  
  // Label.

  void label(const Str &);
  const Str & label() const;

  // Group.
  
  void group(const List<Radio *> &);
  const List<Radio *> & group() const;
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::button::Radio"; }
  
private:

  DJV_FL_WIDGET_CALLBACK(Radio, widget_callback);
  
  void widget_update();

  bool _value;
  Str _label, _label_tmp;
  List<Radio *> _group;
  
  struct _Radio;
  std::auto_ptr<_Radio> _p;
};

//------------------------------------------------------------------------------
// Tool Button

class Tool : public widget::Widget
{
public:

  Tool(const Str & icon, int margin = 2);
  virtual ~Tool();
  virtual void del();
  
  // Value.

  void set(bool = true);
  bool get() const;
  Signal<bool> sig;
  
  // Type.

  enum TYPE { PUSH, TOGGLE, RADIO, REPEAT };
  
  void type(TYPE);
  TYPE type() const;

  void radio_group(const List<Tool *> &);
  const List<Tool *> & radio_group() const;
  
  // Icon.

  void icon(Fl_Image *);
  
  // Shortcut.

  void shortcut(int);
  int shortcut() const;
  
  // Widget.

  virtual int handle(int);
  virtual void draw();
  
  // Layout.
  
  void margin(int);
  int margin() const;

  virtual void dirty();
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::button::Tool"; }

private:

  void action();
  
  DJV_FL_CALLBACK(Tool, repeat_callback);

  bool _value;
  TYPE _type;
  List<Tool *> _radio_group;
  bool _repeat_active;
  Fl_Image * _icon;
  std::auto_ptr<Fl_Image> _icon_inactive;
  int _shortcut;
  int _margin;
  bool _pressed, _pressed_inside;
};

//------------------------------------------------------------------------------
// Functions

void radio_group(const List<Radio *> &);
void radio_group(const List<Tool *> &);

}
}

#endif

