/*------------------------------------------------------------------------------
 include/djv_ui/callback.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_CALLBACK_H
#define DJV_UI_CALLBACK_H

#include <djv_base/base.h>

namespace djv_ui {
using namespace djv_base;

//------------------------------------------------------------------------------
// Callbacks

class Signal_Base;

class Callback_Base
{
public:

  Callback_Base();
  virtual ~Callback_Base() = 0;
  
  void del();
  
  void signal_add(Signal_Base *);
  void signal_del(Signal_Base *);
  uint signal_size() const;
  
  void connect_add(Signal_Base *);
  void connect_del(Signal_Base *);
  uint connect_size() const;
  
  void callbacks(bool);
  bool callbacks() const;
  
private:

  List<Signal_Base *> _signal_list, _connect_list;
  bool _callbacks;
};

#define DJV_CALLBACK(TYPE, NAME, ARG) \
  \
  static void NAME(Callback_Base * that, ARG data) \
  { \
    if (that->callbacks()) \
      static_cast<TYPE *>(that)->NAME(data); \
  } \
  \
  void NAME(ARG in)

//------------------------------------------------------------------------------
// Signals

class Signal_Base
{
public:

  virtual ~Signal_Base();
  
  virtual void release(Callback_Base *) = 0;
  virtual void del() = 0;
};

template <class T>
class Signal : public Signal_Base
{
public:

  inline Signal(Callback_Base * parent);
  inline ~Signal();

  typedef void (Callback)(Callback_Base *, T);
  
  inline void set(Callback_Base *, Callback *);
  inline void del(Callback_Base *);
  inline void del();
  
  inline void emit(T);
  
  inline void release(Callback_Base *);

  inline uint size() const;

private:

  typedef std::pair<Callback_Base *, void *> Pair;
  List<Pair> _list;
  
  Callback_Base * _parent;
};

//------------------------------------------------------------------------------
// FLTK Callbacks

#define DJV_FL_CALLBACK(TYPE, NAME) \
  \
  static void NAME(void * that) \
  { \
    if (static_cast<TYPE *>(that)->callbacks()) \
      static_cast<TYPE *>(that)->NAME(); \
  } \
  \
  void NAME()

#define DJV_FL_WIDGET_CALLBACK(TYPE, NAME) \
  \
  static void NAME(Fl_Widget *, void * that) \
  { \
    if (static_cast<TYPE *>(that)->callbacks()) \
      static_cast<TYPE *>(that)->NAME(); \
  } \
  \
  void NAME()

}

#include <djv_ui/callback_.h>

#endif

