/*------------------------------------------------------------------------------
 include/djv_ui/file_browser.h
 
 File Browser Dialog
 -------------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_FILE_BROWSER_H
#define DJV_UI_FILE_BROWSER_H

#include <djv_base/file.h>
#include <djv_ui/dialog.h>
#include <djv_ui/shortcut.h>

namespace djv_ui {
namespace file_browser {

//------------------------------------------------------------------------------
// Options

enum IMAGE
{
  IMAGE_OFF,
  IMAGE_LOW,
  IMAGE_HIGH
};

enum SORT
{
  SORT_NAME,
  SORT_SIZE,
# if ! defined(WIN32)
  SORT_USER,
# endif
  SORT_TIME,
  _SORT_SIZE
};

//------------------------------------------------------------------------------
// Dialog

class Dialog : public dialog::Base
{
public:

  Dialog();
  ~Dialog();
  
  // Value.
  
  void set(const Str &);
  void dir(const Str &);
  File get() const;
  const Str & dir() const;
  Signal<const File &> sig;
  
  // View.

  void image(IMAGE);
  void seq(djv_base::file::SEQ_COMPRESS);
  void hidden(bool);
  IMAGE image() const;
  djv_base::file::SEQ_COMPRESS seq() const;
  bool hidden() const;
  
  // Sorting.
  
  void sort(SORT);
  void sort_reverse(bool);
  void sort_dir(bool);
  SORT sort() const;
  bool sort_reverse() const;
  bool sort_dir() const;
  
  // Dialog.
  
  void pin(bool);
  bool pin() const;
  
  // Widget.
  
  void show();

  // Layout.
  
  void dirty();

  // Debugging.

  Str dbg_name() const { return "djv::ui::file_browser::Dialog"; }
  
private:

  // Callbacks.

  DJV_CALLBACK(Dialog, widget_callback, const Str &);
  DJV_CALLBACK(Dialog, widget_action_callback, const Str &);
  DJV_CALLBACK(Dialog, dir_callback, const Str &);
  DJV_CALLBACK(Dialog, browser_callback, const File &);
  DJV_CALLBACK(Dialog, browser_action_callback, const File &);
  DJV_CALLBACK(Dialog, columns_callback, const List<int> &);
  DJV_CALLBACK(Dialog, columns_action_callback, int);
  
  DJV_CALLBACK(Dialog, up_callback, bool);
  DJV_CALLBACK(Dialog, prev_callback, bool);
  DJV_CALLBACK(Dialog, current_callback, bool);
#if ! defined(WIN32)
  DJV_CALLBACK(Dialog, home_callback, bool);
#endif
  DJV_CALLBACK(Dialog, desktop_callback, bool);
#if defined(WIN32)
  DJV_CALLBACK(Dialog, drive_callback, bool);
#else
  DJV_CALLBACK(Dialog, root_callback, bool);
#endif
  DJV_CALLBACK(Dialog, reload_callback, bool);
  DJV_FL_WIDGET_CALLBACK(Dialog, _up_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _prev_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _recent_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _current_callback);
#if ! defined(WIN32)
  DJV_FL_WIDGET_CALLBACK(Dialog, _home_callback);
#endif
  DJV_FL_WIDGET_CALLBACK(Dialog, _desktop_callback);
#if defined(WIN32)
  DJV_FL_WIDGET_CALLBACK(Dialog, _drive_callback);
#else
  DJV_FL_WIDGET_CALLBACK(Dialog, _root_callback);
#endif
  DJV_FL_WIDGET_CALLBACK(Dialog, _reload_callback);

  DJV_FL_WIDGET_CALLBACK(Dialog, _image_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _seq_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _hidden_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _sort_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _sort_reverse_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _sort_dir_callback);
  
  DJV_CALLBACK(Dialog, bookmark_del_callback, int);
  DJV_CALLBACK(Dialog, bookmark_del_all_callback, bool);
  DJV_FL_WIDGET_CALLBACK(Dialog, _bookmark_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _bookmark_add_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _bookmark_del_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _bookmark_del_all_callback);
  
  DJV_CALLBACK(Dialog, pin_callback, bool);
  DJV_CALLBACK(Dialog, ok_callback, bool);
  DJV_CALLBACK(Dialog, close_callback, bool);
  
  // Update.

  void dir_update();
  void widget_update();
  void menu_update();
  
  // Variables.

  Str _value, _dir, _dir_prev;
  IMAGE _image;
  djv_base::file::SEQ_COMPRESS _seq;
  bool _hidden;
  SORT _sort;
  bool _sort_reverse, _sort_dir;
  bool _pin;
  
  struct _Dialog;
  std::auto_ptr<_Dialog> _p;
};

//------------------------------------------------------------------------------
// Preferences

class Prefs
{
public:

  Prefs();
  ~Prefs();
  
  // Recent directories.

  void recent(const List<Str> &);
  void recent_add(const Str &);
  const List<Str> & recent() const;
  
  // View.

  void image(IMAGE);
  void seq(djv_base::file::SEQ_COMPRESS);
  void hidden(bool);
  IMAGE image() const;
  djv_base::file::SEQ_COMPRESS seq() const;
  bool hidden() const;
  
  // Sorting.
  
  void sort(SORT);
  void sort_reverse(bool);
  void sort_dir(bool);
  SORT sort() const;
  bool sort_reverse() const;
  bool sort_dir() const;

  // Bookmarks.

  void bookmark(const List<Str> &);
  void bookmark_add(const Str &);
  const List<Str> & bookmark() const;
  
  // Dialog.
  
  void pin(bool);
  bool pin() const;

  // Keyboard shortcuts.
  
  enum SHORTCUT
  {
    UP,
    PREV,
    CURRENT,
    HOME,
    DESKTOP,
    ROOT,
    RELOAD,
    SEQ_OFF,
    SEQ_SPARSE,
    SEQ_RANGE,
    HIDDEN,
    SORT_NAME,
    SORT_SIZE,
#if ! defined(WIN32)
    SORT_USER,
#endif
    SORT_TIME,
    SORT_REVERSE,
    SORT_DIR,
    BOOKMARK_ADD,
    BOOKMARK_1,
    BOOKMARK_2,
    BOOKMARK_3,
    BOOKMARK_4,
    BOOKMARK_5,
    BOOKMARK_6,
    BOOKMARK_7,
    BOOKMARK_8,
    BOOKMARK_9,
    BOOKMARK_10,
    BOOKMARK_11,
    BOOKMARK_12
  };

  void shortcut(const List<shortcut::Shortcut> &);
  const List<shortcut::Shortcut> & shortcut() const;

private:

  List<Str> _recent;
  IMAGE _image;
  djv_base::file::SEQ_COMPRESS _seq;
  bool _hidden;
  SORT _sort;
  bool _sort_reverse, _sort_dir;
  List<Str> _bookmark;
  bool _pin;
  List<shortcut::Shortcut> _shortcut;
};

//------------------------------------------------------------------------------
// Preferences Widget

class Prefs_Widget : public widget::Widget
{
public:

  Prefs_Widget();

private:

  DJV_CALLBACK(Prefs_Widget, shortcut_callback,
    const List<shortcut::Shortcut> &);
};

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, IMAGE &) throw (Str);
Str & operator >> (Str &, SORT &) throw (Str);
Str & operator << (Str &, IMAGE);
Str & operator << (Str &, SORT);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_image();
const List<Str> & label_sort();
const List<Str> & label_shortcut();

extern const Str
  dialog_title,
  dialog_bookmark_del,
  dialog_bookmark_del_all;

extern const Str
  menu_dir,
  menu_dir_up,
  menu_dir_prev,
  menu_dir_recent,
  menu_dir_current,
  menu_dir_home,
  menu_dir_desktop,
  menu_dir_drive,
  menu_dir_root,
  menu_dir_reload,
  menu_view,
  menu_view_image,
  menu_view_seq,
  menu_view_hidden,
  menu_view_sort,
  menu_view_sort_reverse,
  menu_view_sort_dir,
  menu_bookmark,
  menu_bookmark_add,
  menu_bookmark_del,
  menu_bookmark_del_all;

extern const Str
  tooltip_widget,
  tooltip_dir,
  tooltip_up,
  tooltip_prev,
  tooltip_reload;

extern const Str
  prefs_title;

//------------------------------------------------------------------------------
// Global

void set(Callback_Base *, Signal<const File &>::Callback *);
void del(Callback_Base *);

Dialog * global();
Prefs * prefs();

}
}

#endif

