/*------------------------------------------------------------------------------
 include/djv_ui/menu.h
 
 Menu Widgets
 ------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_MENU_H
#define DJV_UI_MENU_H

#include <djv_ui/group.h>

class Fl_Menu_Item;

namespace djv_ui {
namespace menu {

//------------------------------------------------------------------------------
// Constants

static const int item_label_max_size = 100;

//------------------------------------------------------------------------------
// Menu Item

struct Item
{
  Item();
  Item(
    const Str & label,
    int shortcut,
    Fl_Callback * callback,
    void * data,
    int flags
  );
  
  Str label;
  mem::Buffer<char> cstr;
  int shortcut;
  Fl_Callback * callback;
  void * data;
  int flags;
};

//------------------------------------------------------------------------------
// Menu Item Group

class Group
{
public:

  Group();
  Group(const Group &);
  ~Group();
  Group & operator = (const Group &);

  void set(const Group &);
  
  int add(
    const Str & label,
    int shortcut = 0,
    Fl_Callback * = 0,
    void * = 0,
    int flags = 0,
    bool value = false
  );

  List<int> add(
    const List<Str> & label,
    const List<int> & shortcut = List<int>(),
    Fl_Callback * = 0,
    void * = 0,
    int flags = 0,
    int value = -1
  );
  
  void end();
  
  void del();
  
  const List<Item *> & get() const;
  
  void fl(mem::Buffer<Fl_Menu_Item> *) const;

private:

  List<Item *> _list;
};

//------------------------------------------------------------------------------
// Menu Widget

enum
{
  TOGGLE   = 2 << 0,
  RADIO    = 2 << 1,
  VALUE    = 2 << 2,
  SUB_MENU = 2 << 3,
  DIVIDER  = 2 << 4
};

class Widget : public group::Frame
{
public:

  Widget();
  
  // Value.

  void set(const menu::Group &);
  const menu::Group & get() const;

  // Selection.
  
  int item() const;
  bool value() const;
  
  // Widget.
  
  int test_shortcut();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;

private:

  class _Widget;
  
  menu::Group _group;
  _Widget * _widget;
  bool _test_shortcut;
};

//------------------------------------------------------------------------------
// Menu Popup

class Popup : public group::Frame
{
public:

  Popup(const List<Str> & = List<Str>());
  
  // Value.
  
  void set(const List<Str> &);
  void set(const menu::Group &);
  const menu::Group & get() const;
  Signal<int> sig;
  
  // Widget.
  
  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;
  
private:

  class Widget;
  
  DJV_FL_WIDGET_CALLBACK(Popup, widget_callback);

  menu::Group _group;
  Widget * _widget;
  Fl_Image * _icon;
};

//------------------------------------------------------------------------------
// Labels

Str label_list(const Str &, int index);

extern const Str
  menu_messages,
  menu_prefs,
  menu_info,
  menu_about,
  menu_exit;

}
}

#endif
