/*------------------------------------------------------------------------------
 include/djv_ui/num_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <FL/Fl.H>

namespace djv_ui {
namespace num {

template<class T>
inline Base<T>::Base(T min, T max) :
  _value(T()),
  _default_valid(false),
  _default_value(T()),
  _min(min),
  _max(max),
  _clamp(true),
  _inc_small(T()),
  _inc_large(T())
{}

template<class T>
inline Base<T>::~Base()
{}

template<class T>
inline void Base<T>::set(T in)
{
  const T tmp = _clamp ? math::clamp(in, _min, _max) : in;
  if (tmp == _value) return;
  
  _value = tmp;
  
  value_callback();
}

template<class T>
inline const T & Base<T>::get() const { return _value; }
  
template<class T>
inline void Base<T>::default_valid(bool in)
{
  _default_valid = in;
}

template<class T>
inline bool Base<T>::default_valid() const { return _default_valid; }

template<class T>
inline void Base<T>::default_value(T in)
{
  _default_value = in;
  
  default_valid(true);
}

template<class T>
inline const T & Base<T>::default_value() const { return _default_value; }

template<class T>
inline void Base<T>::range(T min, T max)
{
  if (min == _min && max == _max) return;
  
  _min = min;
  _max = max;
  
  set(_value);
  
  range_callback();
}

template<class T>
inline void Base<T>::min_action() { set(_min); }
template<class T>
inline void Base<T>::max_action() { set(_max); }

template<class T>
inline const T & Base<T>::min() const { return _min; }
template<class T>
inline const T & Base<T>::max() const { return _max; }

template<class T>
inline void Base<T>::inc(T inc_small, T inc_large)
{
  if (inc_small == _inc_small && inc_large == _inc_large) return;
  
  _inc_small = inc_small;
  _inc_large = inc_large;
  
  range_callback();
}

template<class T>
inline void Base<T>::inc_small_action() { set(_value + _inc_small); }
template<class T>
inline void Base<T>::inc_large_action() { set(_value + _inc_large); }
template<class T>
inline void Base<T>::dec_small_action() { set(_value - _inc_small); }
template<class T>
inline void Base<T>::dec_large_action() { set(_value - _inc_large); }

template<class T>
inline const T & Base<T>::inc_small() const { return _inc_small; }
template<class T>
inline const T & Base<T>::inc_large() const { return _inc_large; }

template<class T>
inline void Base<T>::clamp(bool in)
{
  _clamp = in;
}

template<class T>
inline int Base<T>::handle(int in)
{
  switch (in)
  {
    case FL_KEYBOARD:
      switch (Fl::event_key()) {
        case FL_Up: inc_small_action(); return 1;
        case FL_Down: dec_small_action(); return 1;
        case FL_Page_Up: inc_large_action(); return 1;
        case FL_Page_Down: dec_large_action(); return 1;
      }
      break;
  }
  return 0;
}

}
}

