/*------------------------------------------------------------------------------
 include/djv_ui/window.h
 
 Window
 ------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_WINDOW_H
#define DJV_UI_WINDOW_H

#include <djv_ui/widget.h>
#include <FL/Fl_Double_Window.H>

namespace djv_ui {
namespace window {

//------------------------------------------------------------------------------
// Window

class Widget : public Fl_Double_Window, public widget::Base
{
public:

  Widget(const Str & title = Str());
  virtual ~Widget();
  virtual void del();
  
  // Title.

  void title(const Str &);
  const Str & title() const;
  
  // Signals.

  Signal<bool> close_sig;
  
  // Widget.

  void fullscreen(bool);
  bool fullscreen() const;

  virtual int handle(int);
  virtual void show();
  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual void update();
  virtual void size(const Vec2i &, bool resizable = true);
  virtual void geom(const Box2i &);
  virtual void resize(int, int, int, int);
  virtual void size(layout::SIZE, layout::SIZE);
  virtual const Box2i & geom() const;
  virtual int visible() const;
  virtual const layout::SIZE * size() const;
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::window::Widget"; }

private:

  DJV_FL_WIDGET_CALLBACK(Widget, close_callback);

  Str _title;
  bool _fullscreen;
  Box2i _geom_tmp;
  bool _resizable;
};

}
}

#endif

