/*------------------------------------------------------------------------------
 lib/djv_base/io_etc.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_base/io.h>

namespace djv_base {
namespace io {

void line(Io * io, char * out, int max_len) throw (Error)
{
  DJV_ASSERT(max_len > 0);
  
  DJV_DBG_OBJ("line");
  
  int i = 0;
  char c = 0;
  do
  {
    io->get(&c, 1);

    if (c != '\n' && c != '\r')
    {
      out[i++] = c;
      DJV_DBG_PRN(Str(out, i));
    }
  }
  while (c != '\n' && c != '\r' && io->valid() && i < (max_len - 1));

  out[i] = 0;
  DJV_DBG_PRN("out = \"" << out << "\"");
}

void word(Io * io, char * out, int max_len) throw (Error)
{
  DJV_ASSERT(max_len > 0);
  
  DJV_DBG_OBJ("word");
  
  out[0] = 0;
  
  enum
  {
    END,
    WORD,
    COMMENT
  } parse = WORD;
  int i = 0;
  while (parse)
  {
    DJV_DBG_PRN("parse = " << parse);
    DJV_DBG_PRN("i = " << i);

    // Get next character.

    uint8_t c;
    io->get(&c, 1);
    DJV_DBG_PRN("c = \'" << c << "\'");
    
    switch (c)
    {
      // Start comment.

      case '#': parse = COMMENT; break;

      // End comment or word.

      case '\0':
      case '\n':
      case '\r': parse = WORD;
      case ' ':
      case '\t': if (out[0]) parse = END; break;

      // Add character to word.

      default:
        if (WORD == parse && i < (max_len - 1))
        {
          out[i++] = c;
          //out[i] = 0;
          //DJV_DBG_PRN("out = \"" << out << "\"");
        }
        break;
    }
  }
  out[i] = 0;
  DJV_DBG_PRN("out = \"" << out << "\"");
}

List<Str> lines(const Str & in)
{
  List<Str> out;
  try
  {
    Io io;
    io.open(in, io::READ);
    while (io.valid())
    {
      char buf [str::cstr_size];
      io::line(&io, buf, str::cstr_size);
      out += buf;
    }
  }
  catch (Error) {}
  return out;
}

const Str
  error = "I/O",
  error_open = "Open",
  error_mmap = "Memory-Map",
  error_read = "Read",
  error_write = "Write",
  error_position = "Position";

}
}

