/*------------------------------------------------------------------------------
 lib/djv_image/cgl.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#if defined(DJV_MACOSX)

//#define DJV_DBG

#include "cgl.h"

namespace djv_image {
namespace gl {

namespace {

class _Pixel_Format
{
public:

  _Pixel_Format() :
    _format(0)
  {
    DJV_DBG_OBJ("_Pixel_Format::_Pixel_Format");
  }

  ~_Pixel_Format()
  {
    if (_format) CGLDestroyPixelFormat(_format);
  }

  void init() throw (Error)
  {
    DJV_DBG_OBJ("_Pixel_Format::init");

    CGLPixelFormatAttribute attribs [] =
    {
      //kCGLPFAColorSize, (CGLPixelFormatAttribute)32,
      (CGLPixelFormatAttribute)0
    };

    long npix = 0;
    CGLError error = CGLChoosePixelFormat(attribs, &_format, &npix);
    if (error != kCGLNoError)
      throw Error(error_cgl, str::Format(error_cgl_pixel_format).arg(error));
  }

  CGLPixelFormatObj format() const { return _format; }

private:

  CGLPixelFormatObj _format;
};

}

_Cgl_Device::_Cgl_Device() :
  _context(0)
{
  DJV_DBG_OBJ("_Cgl_Device::_Cgl_Device");
}

_Cgl_Device::~_Cgl_Device()
{
  DJV_DBG_OBJ("_Cgl_Device::~_Cgl_Device");

  if (_context)
  {
    DJV_DBG_PRN("context");
    CGLDestroyContext(_context);
  }
}

void _Cgl_Device::init() throw (Error)
{
  DJV_DBG_OBJ("_Cgl_Device::init");

  // XXX Create a dummy rendering context for glewInit.
  // According to the docs, glewInit can be called just once per-process?

  CGLError error = kCGLNoError;

  _Pixel_Format pixel_format;
  pixel_format.init();

  error = CGLCreateContext(pixel_format.format(), 0, &_context);
  if (error != kCGLNoError || ! _context)
    throw Error(error_cgl, str::Format(error_cgl_context_create).arg(error));

  error = CGLSetCurrentContext(_context);
  if (error != kCGLNoError)
    throw Error(error_cgl, str::Format(error_cgl_context_bind).arg(error));

  // Initialize GLEW.

  GLint gl_error = glewInit();
  if (gl_error != GLEW_OK)
    throw Error(error_glew, str::Format(error_glew_init).
      arg((char *)glewGetErrorString(gl_error)));

  _vendor_str = Str((const char *)glGetString(GL_VENDOR));
  _render_str = Str((const char *)glGetString(GL_RENDERER));
  _version_str = Str((const char *)glGetString(GL_VERSION));
  DJV_DBG_PRN("OpenGL vendor string = " << _vendor_str);
  DJV_DBG_PRN("OpenGL renderer string = " << _render_str);
  DJV_DBG_PRN("OpenGL version string = " << _version_str);
  DJV_DBG_PRN("OpenGL extensions = " <<
    (const char *)glGetString(GL_EXTENSIONS));
  DJV_DBG_PRN("glu version = " <<
    (const char *)gluGetString(GLU_VERSION));
  DJV_DBG_PRN("glu extensions = " <<
    (const char *)gluGetString(GLU_EXTENSIONS));
}

Global::Global() throw (Error) :
  _device(new _Cgl_Device),
  _pipeline(PIPELINE(0))
{
  DJV_DBG_OBJ("Global::Global");
  
  _device->init();
  
  const Str & version = _device->version_str();
  if (version.size())
    if ('2' == version[0])
      _pipeline = PROGRAM;
  DJV_DBG_PRN("pipeline = " << _pipeline);
}

Global::~Global() {}

}
}

#endif
