/*------------------------------------------------------------------------------
 lib/djv_image/gl_data_copy.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/gl_data.h>

namespace djv_image {
namespace gl_data {

void copy(
  const djv_image::Data & in,
  djv_image::Data * out,
  const Option & option,
  Data * data,
  gl_offscreen::Buffer * buffer
) throw (Error) {

  DJV_ASSERT(out);
  
  DJV_DBG_OBJ("copy");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("out = " << *out);

  const Vec2i & size = out->info().size;

  std::auto_ptr<gl_offscreen::Buffer> _buffer;
  if (! buffer)
  {
    DJV_DBG_PRN("bufffer create");
    
    _buffer = std::auto_ptr<gl_offscreen::Buffer>(
      gl_offscreen::global()->buffer_create());
    buffer = _buffer.get();
    buffer->init(Info(size, out->pixel()));
  }
  buffer->bind();

  gl::ortho(size);
  DJV_DBG_GL(glViewport(0, 0, size.x, size.y));
  
  //DJV_DBG_GL(glClearColor(0, 1, 0, 0));
  djv_image::Color background(djv_image::pixel::RGB_F32);
  djv_image::color::convert(option.background, &background);
  DJV_DBG_GL(glClearColor(
    background.get_f32(0),
    background.get_f32(1),
    background.get_f32(2),
    0.0));
  DJV_DBG_GL(glClear(GL_COLOR_BUFFER_BIT));
  
  Option _option = option;
  if (out->info().mirror.x) _option.xform.mirror.x = ! _option.xform.mirror.x;
  if (out->info().mirror.y) _option.xform.mirror.y = ! _option.xform.mirror.y;
  const pixel::TYPE type = pixel::type(out->info().pixel);
  _option.clamp = ! (pixel::F16 == type || pixel::F32 == type);

  Data _data;
  Data * data_p = data ? data : &_data;
  data_p->draw(in, _option);
  data_p->read(out);

  buffer->unbind();
}

}
}
