/*------------------------------------------------------------------------------
 lib/djv_image/io_cineon_info.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef _DJV_IMAGE_IO_CINEON_INFO_H
#define _DJV_IMAGE_IO_CINEON_INFO_H

#include <djv_image/io_cineon.h>

namespace djv_image {
namespace io_cineon {
namespace info {

//------------------------------------------------------------------------------
// Constants

const uint32_t magic [] =
{
  0x802a5fd7,
  0xd75f2a80
};

const float speed_min = 0.000001; // XXX

//------------------------------------------------------------------------------
// Options

enum ORIENT
{
  ORIENT_LEFT_RIGHT_TOP_BOTTOM,
  ORIENT_LEFT_RIGHT_BOTTOM_TOP,
  ORIENT_RIGHT_LEFT_TOP_BOTTOM,
  ORIENT_RIGHT_LEFT_BOTTOM_TOP,
  ORIENT_TOP_BOTTOM_LEFT_RIGHT,
  ORIENT_TOP_BOTTOM_RIGHT_LEFT,
  ORIENT_BOTTOM_TOP_LEFT_RIGHT,
  ORIENT_BOTTOM_TOP_RIGHT_LEFT
};

enum DESCRIPTOR
{
  DESCRIPTOR_L = 0,
  DESCRIPTOR_R_PRINT = 1,
  DESCRIPTOR_G_PRINT = 2,
  DESCRIPTOR_B_PRINT = 3,
  DESCRIPTOR_R_CCIR_XA11 = 4,
  DESCRIPTOR_G_CCIR_XA11 = 5,
  DESCRIPTOR_B_CCIR_XA11 = 6
};

//------------------------------------------------------------------------------
// Header

struct Header
{
  Header();
  
  void load(djv_base::io::Io *, Info *, Tag *, time::Speed *, bool *)
    throw (Error);
  void save(djv_base::io::Io *, const Str & file, const Info &, const Tag &,
    COLOR) throw (Error);
  
  void endian();
  
  void dbg() const;

private:

  struct File
  {
    uint32_t magic;
    uint32_t image_offset;
    uint32_t header_size;
    uint32_t industry_header_size;
    uint32_t user_header_size;
    uint32_t size;
    char version [8];
    char name [100];
    char time [24];
    uint8_t _pad [36];
  } _file;
  
  struct Image
  {
    uint8_t orient;
    uint8_t channels;
    uint8_t _pad [2];

    struct Channel
    {
      uint8_t descriptor [2];
      uint8_t bit_depth;
      uint8_t _pad;
      uint32_t size [2];
      float low_data;
      float low_quantity;
      float high_data;
      float high_quantity;
    } channel [8];
    
    float white [2];
    float red [2];
    float green [2];
    float blue [2];
    char label [200];
    uint8_t _pad2 [28];
    uint8_t interleave;
    uint8_t packing;
    uint8_t data_sign;
    uint8_t data_sense;
    uint32_t line_padding;
    uint32_t channel_padding;
    uint8_t _pad3 [20];
  } _image;
  
  struct Source
  {
    int32_t offset [2];
    char file [100];
    char time [24];
    char input_device [64];
    char input_model [32];
    char input_serial [32];
    float input_pitch [2];
    float gamma;
    char _pad [40];
  } _source;
  
  struct Film
  {
    uint8_t id;
    uint8_t type;
    uint8_t offset;
    uint8_t _pad;
    uint32_t prefix;
    uint32_t count;
    char format [32];
    uint32_t frame;
    float frame_rate;
    char frame_id [32];
    char slate [200];
    char _pad2 [740];
  } _film;
};

//------------------------------------------------------------------------------
// Functions

void zero(int32_t *);
void zero(float *);
void zero(char *, int size);

bool valid(const uint8_t *);
bool valid(const uint16_t *);
bool valid(const uint32_t *);
bool valid(const int32_t *);
bool valid(const float *);
bool valid(const char *, int size);

Str to_str(const char *, int size);

//------------------------------------------------------------------------------
// Debugging

Str dbg(uint8_t);
Str dbg(uint16_t);
Str dbg(uint32_t);
Str dbg(int32_t);
Str dbg(float);
Str dbg(const char *, int size);

}
}
}

#endif
