/*------------------------------------------------------------------------------
 lib/djv_image/io_dpx_save.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_dpx.h>

namespace djv_image {
namespace io_dpx {

io::Plugin * Save::copy() const
{
  Save * out = new Save;
  out->_option = _option;
  return out;
}

const Str & Save::name() const { return io_dpx::name; }

const List<Str> & Save::extension_list() const
  { return io_dpx::extension_list; }

bool Save::option(const Str & in, Str * data)
{
try
{
  if (str::compare(in, option_list()[SAVE_COLOR])) *data >> _option.color;
  else if (str::compare(in, option_list()[SAVE_PRINT])) *data >> _option.print;
  else if (str::compare(in, option_list()[SAVE_TYPE])) *data >> _option.type;
  else if (str::compare(in, option_list()[SAVE_ENDIAN]))
    *data >> _option.endian;
}
catch (Str) { return false; }
  return true;
}

Str Save::option(const Str & in) const
{
  Str out;
  if (str::compare(in, option_list()[SAVE_COLOR])) out << _option.color;
  else if (str::compare(in, option_list()[SAVE_PRINT])) out << _option.print;
  else if (str::compare(in, option_list()[SAVE_TYPE])) out << _option.type;
  else if (str::compare(in, option_list()[SAVE_ENDIAN])) out << _option.endian;
  return out;
}

const List<Str> & Save::option_list() const { return label_save_option(); }

void Save::open(const File & in, const Info & info, const Tag &,
  const Time & time) throw (Error)
{
  DJV_DBG_OBJ("Save::open");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("info = " << info);
  
  _file = in;
  if (time.list.size() > 1) _file.type = file::SEQ;
  
  _info = Info();
  _info.size = info.size;
  _info.mirror.y = true;
  switch (_option.endian)
  {
    case ENDIAN_AUTO: break;
    case MSB: _info.endian = mem::MSB; break;
    case LSB: _info.endian = mem::LSB; break;
  }
  switch (_option.type)
  {
    case U10: _info.pixel = pixel::RGB_U10; break;
    default:
    {
      pixel::TYPE type = pixel::type(info.pixel);
      switch (type)
      {
        case pixel::F16: 
        case pixel::F32: type = pixel::U16; break;
        default: break;
      }
      _info.pixel = pixel::pixel(pixel::format(info.pixel), type);
    } break;
  }
  switch (pixel::bit_depth(_info.pixel))
  {
    case 8:
    case 10: _info.align = 4; break;
    default: break;
  }
  DJV_DBG_PRN("info = " << _info);

  _image.set(_info);
}

void Save::_open(const File & in, const Tag & tag) throw (Error)
{
  DJV_DBG_OBJ("Save::_open");
  DJV_DBG_PRN("in = " << in);

  _io.open(in, djv_base::io::WRITE);
  
  info_save(&_io, in, _info, tag, _option.endian, _option.color);
}

void Save::set(const Image & in, int frame) throw (Error)
{
  DJV_DBG_OBJ("Save::set");
  DJV_DBG_PRN("in = " << in);
  
  // Setup.
  
  Profile profile;
  if (io_cineon::COLOR_PRINT == _option.color ||
    io_cineon::COLOR_AUTO == _option.color)
  {
    DJV_DBG_PRN("profile");
    profile.type = profile::LUT;
    profile.lut = print_lut(_option.print);
  }

  // Open.
  
  _open(_file.get(frame), in.tag);
  
  // Write.

  const Data * p = &in;
  if (
    in.info() != _info ||
    in.profile.type != profile::RAW ||
    profile.type != profile::RAW
  ) {
    DJV_DBG_PRN("convert = " << _image);
    
    _image.zero();
    
    gl_data::Option option;
    option.profile = profile;
    gl_data::copy(*p, &_image, option);
    
    p = &_image;
  }
  
  _io.set(p->data(), p->bytes_data());
  
  info_update(&_io);
  _io.close();
}

}
}

