/*------------------------------------------------------------------------------
 lib/djv_image/io_rla_data.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_rla.h>

namespace djv_image {
namespace io_rla {

void rle_load(
  djv_base::Io * in,
  uint8_t * out,
  int size,
  int channels,
  int bytes
) throw (Error) {
  DJV_DBG_OBJ("rle_load");
  DJV_DBG_PRN("size = " << size);
  DJV_DBG_PRN("channels = " << channels);
  DJV_DBG_PRN("bytes = " << bytes);
  
  int16_t _size = 0;
  in->get_16(&_size);
  DJV_DBG_PRN("io size = " << _size);
  
  const uint8_t * start = in->mmap_p();
  const uint8_t * p = start;
  in->seek(_size);
  
  for (int b = 0; b < bytes; ++b)
  {
    uint8_t * out_p = out + (mem::LSB == mem::endian() ? (bytes - 1 - b) : b);
    const int out_inc = channels * bytes;
    
    for (int i = 0; i < size;)
    {
      int count = *((int8_t *)p);
      ++p;
      DJV_DBG_PRN("count = " << count);

      if (count >= 0)
      {
        ++count;
        for (int j = 0; j < count; ++j, out_p += out_inc)
          *out_p = *p;
        ++p;
      }
      else
      {
        count = -count;
        for (int j = 0; j < count; ++j, ++p, out_p += out_inc)
          *out_p = *p;
      }
    
      i += count;
    }
  }
  
  DJV_DBG_PRN("out = " << p - start);
}

void float_load(
  djv_base::Io * in,
  uint8_t * out,
  int size,
  int channels
) throw (Error) {
  DJV_DBG_OBJ("float_load");
  DJV_DBG_PRN("size = " << size);
  DJV_DBG_PRN("channels = " << channels);
  
  int16_t _size = 0;
  in->get_16(&_size);
  DJV_DBG_PRN("io size = " << _size);
  
  const uint8_t * start = in->mmap_p();
  const uint8_t * p = start;
  in->seek(_size);
  
  const int out_inc = channels * 4;
  if (mem::LSB == mem::endian())
    for (int i = 0; i < size; ++i, p += 4, out += out_inc)
    {
      out[0] = p[3];
      out[1] = p[2];
      out[2] = p[1];
      out[3] = p[0];
    }
  else
    for (int i = 0; i < size; ++i, p += 4, out += out_inc)
    {
      out[0] = p[0];
      out[1] = p[1];
      out[2] = p[2];
      out[3] = p[3];
    }

  DJV_DBG_PRN("out = " << p - start);
}

void skip(djv_base::Io * in) throw (Error)
{
  int16_t size = 0;
  in->get_16(&size);
  in->seek(size);
}

}
}
