/*------------------------------------------------------------------------------
 lib/djv_image/tag.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_image/tag.h>

namespace djv_image {
namespace tag {
  
void Tag::add(const Tag & in)
{
  for (uint i = 0; i < in._list.size(); ++i)
    add(in._list[i].first, in._list[i].second);
}

void Tag::add(const Str & key, const Str & value)
{
  for (uint i = 0; i < _list.size(); ++i)
    if (key == _list[i].first)
    {
      _list[i].second = value;
      return;
    }
  
  _list += Pair(key, value);
}

Str & Tag::operator [] (const Str & key)
{
  for (uint i = 0; i < _list.size(); ++i)
    if (key == _list[i].first)
      return _list[i].second;

  _list += Pair(key, Str());
  return _list[_list.size() - 1].second;
}

Str Tag::get(const Str & key) const
{
  for (uint i = 0; i < _list.size(); ++i)
    if (key == _list[i].first)
      return _list[i].second;
  
  return Str();
}

Str Tag::operator [] (const Str & key) const { return get(key); }

List<Str> Tag::key_list() const
{
  List<Str> out(_list.size());
  for (uint i = 0; i < _list.size(); ++i)
    out[i] = _list[i].first;

  return out;
}

List<Str> Tag::value_list() const
{
  List<Str> out(_list.size());
  for (uint i = 0; i < _list.size(); ++i)
    out[i] = _list[i].second;

  return out;
}

bool Tag::valid(const Str & key)
{
  for (uint i = 0; i < _list.size(); ++i)
    if (key == _list[i].first)
      return true;

  return false;
}

void Tag::del()
{
  _list.clear();
}

bool operator == (const Tag & a, const Tag & b)
{
  if (a.key_list() != b.key_list()) return false;
  if (a.value_list() != b.value_list()) return false;
  return true;
}

bool operator != (const Tag & a, const Tag & b)
{
  return ! (a == b);
}

const List<Str> & label_tag()
{
  static const List<Str> data = List<Str>() <<
    "Project" <<
    "Creator" <<
    "Description" <<
    "Copyright" <<
    "Time" <<
    "UTC Offset" <<
    "Keycode" <<
    "Timecode";
  return data;
}

}
}
