/*------------------------------------------------------------------------------
 lib/djv_ui/button_push.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/button.h>
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Button.H>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace button {

namespace {

class _Button : public Fl_Button
{
public:

  _Button() : Fl_Button(0, 0, 0, 0)
  {
    box(FL_THIN_UP_BOX);
    down_box(FL_THIN_DOWN_BOX);
  }
};

}

struct Push::_Push
{
  mem::Buffer<char> label;
  _Button * widget;
};

Push::Push(const Str & in) :
  sig(this),
  _p(new _Push)
{
  //text_font(style::global()->font_bold());

  Fl_Group::add(_p->widget = new _Button);
  
  Item::size(layout::FIXED, layout::FIXED);
  
  label(in);
  
  _p->widget->callback(widget_callback, this);
}

Push::~Push()
{}

void Push::label(const Str & in)
{
  if (in == _label) return;

  _label = in;
  
  const int size = _label.size() + 1;
  _p->label.size(size);
  str::cstr(_label, _p->label(), size);
  _p->widget->label(_p->label());
}

const Str & Push::label() const { return _label; }

void Push::dirty()
{
  DJV_DBG_OBJ("Push::dirty");
  
  widget::Widget::dirty();

  _p->widget->labelfont(text_font());
  _p->widget->labelsize(style::global()->font_size());
  _p->widget->labelcolor(text_color());
  
  fl_font(text_font(), style::global()->font_size());
  const Vec2i label_size(_p->label() ? base::str_size(_p->label()) :
    Vec2i(fl_height()));

  size_hint(Vec2i(math::max(label_size.x, style::global()->size_button()),
    label_size.y) +
    style::global()->margin_widget() * 2 +
    Fl::box_dx(_p->widget->box()) * 2);
}

void Push::geom(const Box2i & in)
{
  widget::Widget::geom(in);
  _p->widget->resize(in.x, in.y, in.w, in.h);
}

const Box2i & Push::geom() const { return widget::Widget::geom(); }

void Push::widget_callback()
{
  sig.emit(true);
}

}
}

