/*------------------------------------------------------------------------------
 lib/djv_ui/button_radio.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Round_Button.H>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace button {

namespace {

class _Button : public Fl_Round_Button
{
public:

  _Button() :
    Fl_Round_Button(0, 0, 0, 0)
  {}

  int handle(int in)
  {
    switch (in)
    {
      case FL_SHORTCUT:
        if (! (shortcut() ? Fl::test_shortcut(shortcut()) : test_shortcut()))
          return 0;
        value(! value());
        do_callback();
        return 1;
    }
    return Fl_Round_Button::handle(in);
  }
};

}

struct Radio::_Radio
{
  mem::Buffer<char> label;
  _Button * widget;
};

Radio::Radio(const Str & in) :
  sig(this),
  _value(false),
  _p(new _Radio)
{
  //text_font(style::global()->font_bold());

  Fl_Group::add(_p->widget = new _Button);
  
  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  label(in);
  
  _p->widget->callback(widget_callback, this);
}

Radio::~Radio()
{}
  
void Radio::set(bool in)
{
  if (in == _value) return;
  
  _value = in;
  
  widget_update();
  sig.emit(_value);
}

bool Radio::get() const { return _value; }

void Radio::label(const Str & in)
{
  if (in == _label) return;
  
  _label = in;
  _label_tmp = " " + _label; // XXX Add spacing.
  
  const int size = _label_tmp.size() + 1;
  _p->label.size(size);
  str::cstr(_label_tmp, _p->label(), size);
  _p->widget->label(_p->label());
}

const Str & Radio::label() const { return _label; }

void Radio::group(const List<Radio *> & in)
{
  _group = in;
}

const List<Radio *> & Radio::group() const { return _group; }

void Radio::dirty()
{
  widget::Widget::dirty();

  _p->widget->labelfont(text_font());
  _p->widget->labelsize(style::global()->font_size());
  _p->widget->labelcolor(text_color());
  
  fl_font(text_font(), style::global()->font_size());
  const Vec2i label_size(
    _p->label() ? base::str_size(_p->label()) : Vec2i(fl_height()));
  
  size_hint(
    Vec2i(fl_height() + label_size.x, label_size.y) +
    style::global()->margin_text() * 2);
}

void Radio::geom(const Box2i & in)
{
  widget::Widget::geom(in);
  _p->widget->resize(in.x, in.y, in.w, in.h);
}

const Box2i & Radio::geom() const { return widget::Widget::geom(); }

void Radio::widget_callback()
{
  if (! _value) set(_p->widget->value() ? true : false);
  widget_update();
}

void Radio::widget_update()
{
  callbacks(false);
  
  _p->widget->value(_value);
  
  if (_value)
  {
    for (uint i = 0; i < _group.size(); ++i)
    if (_group[i] != this) _group[i]->set(false);
  }
  
  callbacks(true);
}

}
}
