/*------------------------------------------------------------------------------
 lib/djv_ui/color_swatch.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/dialog.h>
#include <djv_ui/color.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/style.h>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace color {

Swatch::Swatch() :
  sig(this),
  push_sig(this)
{
  frame(FL_THIN_DOWN_BOX);
  
  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::LEFT);
}

Swatch::~Swatch()
{}

void Swatch::set(const djv_image::Color & in)
{
  if (in == _value) return;
  
  _value = in;
  
  redraw();
  sig.emit(_value);
}

const djv_image::Color & Swatch::get() const { return _value; }

int Swatch::handle(int in)
{
  switch (in)
  {
    case FL_PUSH: push_sig.emit(true); return 1;
  }
  return Frame::handle(in);
}

void Swatch::draw()
{
  Frame::draw();

  const Box2i & geom = frame_geom();

  djv_image::Color tmp(djv_image::pixel::RGB_U8);
  djv_image::color::convert(_value, &tmp);
  fl_color(tmp.get_u8(0), tmp.get_u8(1), tmp.get_u8(2));
  fl_rectf(geom.x, geom.y, geom.w, geom.h);
}

void Swatch::dirty()
{
  Frame::dirty();

  size_hint(Vec2i(style::global()->size_swatch()) + frame_size() * 2);
}

struct Swatch_Widget::_Swatch_Widget
{
  Swatch * widget;
};

Swatch_Widget::Swatch_Widget() :
  sig(this),
  _p(new _Swatch_Widget)
{
  _p->widget = new Swatch;

  size(layout::FIXED, layout::FIXED);

  layout::Horizontal * layout = new Layout_H(this);
  layout->margin(0);
  layout->add(_p->widget);
  
  _p->widget->sig.set(this, widget_callback);
  _p->widget->push_sig.set(this, dialog_callback);
}

Swatch_Widget::~Swatch_Widget()
{}

void Swatch_Widget::set(const djv_image::Color & in)
{
  _p->widget->set(in);
}

const djv_image::Color & Swatch_Widget::get() const
  { return _p->widget->get(); }

void Swatch_Widget::widget_callback(const djv_image::Color & in)
{
  _p->widget->set(in);
  
  sig.emit(in);
}

void Swatch_Widget::dialog_callback(bool)
{
  dialog::global()->color(get(), this, widget_callback);
}

}
}
