/*------------------------------------------------------------------------------
 lib/djv_ui/dialog_choice.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/dialog.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/style.h>

namespace djv_ui {
namespace dialog {

struct Choice::_Choice
{
  choice::Widget * widget;
  label::Widget * label;
};

Choice::Choice(const Str & label, const List<Str> & list) :
  Message_Base(label_choice),
  sig(this),
  _p(new _Choice)
{
  // Widgets.

  _p->widget = new choice::Widget(list);
  _p->label = new label::Widget(label);
  _p->widget->label(_p->label);

  button::Push * ok_button = new button::Push(label_ok);
  button::Push * close_button = new button::Push(label_close);

  // Layout.

  Layout_V * layout = new Layout_V(this);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout->stretch(layout_h);
  layout_h->margin(style::global()->margin_large());
  layout_h->add_stretch(false);
  layout_h->add(_p->widget);
  layout_h->add_stretch(false);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add_stretch();
  layout_h->add(ok_button);
  layout_h->add(close_button);
  
  // Initialize.

  size(size_hint(), false);

  // Callbacks.
  
  ok_button->sig.set(this, ok_callback);
  close_button->sig.set(this, close_callback);
}

Choice::~Choice()
{}

void Choice::list(const List<Str> & in)
{
  _p->widget->list(in);
  
  size(size_hint(), false);
}

List<Str> Choice::list() const { return _p->widget->list(); }

void Choice::label(const Str & in)
{
  _p->label->set(in);
  
  size(size_hint(), false);
}

const Str & Choice::label() const { return _p->label->get(); }

void Choice::show()
{
  _p->widget->take_focus();

  Message_Base::show();
}

void Choice::ok_callback(bool)
{
  sig.emit(_p->widget->get());
  
  hide();
}

void Choice::close_callback(bool)
{
  hide();
}

}
}

