/*------------------------------------------------------------------------------
 lib/djv_ui/dialog_message.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/dialog.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/style.h>

namespace djv_ui {
namespace dialog {

Message_Base::Message_Base(const Str & title) :
  Base(title)
{}

Message_Base::~Message_Base()
{}

void Message_Base::dirty()
{
  Base::dirty();
  
  _size_min = max(Base::size_min(), Vec2i(300, 100));
}

const Vec2i & Message_Base::size_min() const { return _size_min; }

struct Message::_Message
{
  label::Widget * widget;
  button::Push * close;
};

Message::Message(const Str & label) :
  Message_Base(label_message),
  _p(new _Message)
{
  // Widgets.

  _p->widget = new label::Widget(label);
  
  _p->close = new button::Push(label_close);

  // Layout.

  Layout_V * layout = new Layout_V(this);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout->stretch(layout_h);
  layout_h->margin(style::global()->margin_large());
  layout->add(_p->widget);
  layout->stretch(_p->widget);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add_stretch();
  layout_h->add(_p->close);
  
  // Initialize.

  size(size_hint());
  
  // Callbacks.

  _p->close->sig.set(this, close_callback);
}

Message::~Message()
{}

void Message::label(const Str & in)
{
  _p->widget->set(in);
  
  size(size_hint());
}

const Str & Message::label() const { return _p->widget->get(); }

void Message::show()
{
  _p->close->take_focus();

  Message_Base::show();
}

void Message::close_callback(bool)
{
  hide();
}

}
}

