/*------------------------------------------------------------------------------
 lib/djv_ui/file.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/file.h>
#include <djv_ui/file_browser.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/text.h>

namespace djv_ui {
namespace file {

struct Widget::_Widget
{
  text::Edit * widget;
};

Widget::Widget() :
  sig(this),
  _p(new _Widget)
{
  // Widgets.
  
  _p->widget = new text::Edit;
  
  button::Push * dialog_widget = new button::Push(label_browse);
  
  // Layout.
  
  Layout_H * layout = new Layout_H(this);
  layout->margin(0);
  
  layout->add(_p->widget);
  layout->stretch(_p->widget);
  
  layout->add(dialog_widget);
  
  // Initialize.
  
  widget_update();

  // Callbacks.
  
  _p->widget->sig.set(this, widget_callback);
  dialog_widget->sig.set(this, dialog_callback);
}

Widget::~Widget()
{}

void Widget::set(const File & in)
{
  if (in == _value) return;
  
  _value = in;
  
  widget_update();
  sig.emit(_value);
}

const File & Widget::get() const { return _value; }

void Widget::widget_callback(const Str & in) { set(in); }

void Widget::dialog_callback(bool) { file_browser::set(this, dialog_callback); }
void Widget::dialog_callback(const File & in) { set(in); }

void Widget::widget_update() { _p->widget->set(_value); }

void recent(const Str & in, List<Str> * list, int max)
{
  List<Str>::iterator i = std::find(list->begin(), list->end(), in);
  
  if (i == list->end())
  {
    // Insert new item at front of list.
    
    list->insert(list->begin(), in);
    
    while (int(list->size()) > max)
      list->erase(list->end() - 1);
  }
  else
  {
    // Move existing item to front of list.
    
    const Str tmp = *i;
    list->erase(i);
    list->insert(list->begin(), tmp);
  }
}

const Str
  label_browse = "&Browse";

}
}

