/*------------------------------------------------------------------------------
 lib/djv_ui/group_box.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/group.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/style.h>
#include <FL/Fl_Group.H>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace group {

Box::Box(const Str & label) :
  _label(label)
{
  text_font(style::global()->font_bold());
  
  new Layout_V(this);
}

Box::~Box()
{}

void Box::del()
{
  widget::Widget::del();
}

void Box::label(const Str & in)
{
  if (in == _label) return;
  
  _label = in;
  
  redraw();
}

const Str & Box::label() const { return _label; }

void Box::draw()
{
  const Box2i & geom = this->geom();
  const int margin = style::global()->margin_widget();
  
  if (_label.empty())
  {
    // Draw box.
  
    fl_draw_box(
      FL_ENGRAVED_FRAME,
      geom.x,
      geom.y,
      geom.w,
      geom.h,
      FL_BACKGROUND_COLOR);
  }
  else
  {
    // Draw box.
  
    fl_draw_box(
      FL_ENGRAVED_FRAME,
      geom.x,
      geom.y + _size_label.y / 2,
      geom.w,
      geom.h - _size_label.y / 2,
      FL_BACKGROUND_COLOR);
  
    // Draw label.

    fl_font(text_font(), style::global()->font_size());
  
    fl_color(FL_BACKGROUND_COLOR);
    //fl_color(FL_BLUE);
    fl_rectf(
      geom.x + 2 + margin,
      geom.y,
      _size_label.x + margin * 2,
      _size_label.y);
  
    fl_color(FL_FOREGROUND_COLOR);
    fl_draw(
      _label.c_str(),
      geom.x + 2 + margin * 2,
      geom.y + fl_height() - fl_descent());
  }

  widget::Widget::draw();
}

void Box::dirty()
{
  DJV_DBG_OBJ("Box::dirty");
  
  widget::Widget::dirty();

  fl_font(text_font(), style::global()->font_size());

  const Vec2i size = layout() ? layout()->size_hint() : Vec2i();
  DJV_DBG_PRN("size = " << size);

  _size_label = _label.empty() ?
    Vec2i(0, 2) :
    Vec2i(base::str_width(_label), fl_height());
  DJV_DBG_PRN("label size = " << _size_label);
  
  const int margin = style::global()->margin_widget();
  size_hint(Vec2i(
    math::max(_size_label.x + margin * 4, size.x) + 2 * 2,
    _size_label.y + size.y + 2));
}

void Box::geom(const Box2i & in)
{
  DJV_DBG_OBJ("Box::geom");
  DJV_DBG_PRN("in = " << in);
  
  //Fl_Group::resize(in.x, in.y, in.w, in.h);
  Fl_Widget::resize(in.x, in.y, in.w, in.h);
  layout::Item::geom(in);
  
  if (layout())
    layout()->geom(Box2i(
      in.x + 2,
      in.y + _size_label.y,
      in.w - 2 * 2,
      in.h - _size_label.y - 2));
}

const Box2i & Box::geom() const { return widget::Widget::geom(); }

}
}
