/*------------------------------------------------------------------------------
 lib/djv_ui/image_io_dpx.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/image_io_cineon.h>
#include <djv_ui/image_io_dpx.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/num_int.h>
#include <djv_ui/num_float.h>
#include <djv_ui/style.h>

namespace djv_ui {
namespace image_io_dpx {
using namespace djv_image::io_dpx;

struct Load::_Load
{
  choice::Widget * color;
  num::Int_Slider * black, * white;
  num::Float_Slider * gamma;
  num::Int_Slider * soft_clip;
};

Load::Load() :
  _plugin(0),
  _p(new _Load)
{
  DJV_DBG_OBJ("Load::Load");
  
  // Widgets.
  
  group::Box * color_group =
    new group::Box(image_io_cineon::label_color_group);
  _p->color =
    new choice::Widget(djv_image::io_cineon::label_color(),
      image_io_cineon::label_color);
  
  group::Box * print_group =
    new group::Box(image_io_cineon::label_print_group);
  _p->black =
    new num::Int_Slider(0, 1023, image_io_cineon::label_print_black);
  _p->white =
    new num::Int_Slider(0, 1023, image_io_cineon::label_print_white);
  _p->gamma =
    new num::Float_Slider(0.01, 4.0, image_io_cineon::label_print_gamma);
  _p->soft_clip =
    new num::Int_Slider(0, 50, image_io_cineon::label_print_soft_clip);
  
  // Layout.
  
  layout::group(List<layout::Item *>() <<
    _p->black <<
    _p->white <<
    _p->gamma <<
    _p->soft_clip);
  
  Layout_V * layout, * layout_group, * layout_v;
  
  layout = new Layout_V(this);

  layout->add(color_group);
  layout_group = new Layout_V(color_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->color);
  
  layout->add(print_group);  
  layout_group = new Layout_V(print_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  layout_v->add(_p->black);
  layout_v->add(_p->white);
  layout_v->add(_p->gamma);
  layout_v->add(_p->soft_clip);
  
  layout->add_stretch();

  // Initialize.

  _p->black->default_value(Load_Option().print.black);
  _p->white->default_value(Load_Option().print.white);
  _p->gamma->default_value(Load_Option().print.gamma);
  _p->soft_clip->default_value(Load_Option().print.soft_clip);
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->color->sig.set(this, color_callback);
  _p->black->sig.set(this, black_callback);
  _p->white->sig.set(this, white_callback);
  _p->gamma->sig.set(this, gamma_callback);
  _p->soft_clip->sig.set(this, soft_clip_callback);
}

Load::~Load()
{}

plugin::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return djv_image::io_dpx::name; }

void Load::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Load::color_callback(int in)
  { callback((_option.color = djv_image::io_cineon::COLOR(in))); }
void Load::black_callback(int in)
  { callback((_option.print.black = in)); }
void Load::white_callback(int in)
  { callback((_option.print.white = in)); }
void Load::gamma_callback(double in)
  { callback((_option.print.gamma = in)); }
void Load::soft_clip_callback(int in)
  { callback((_option.print.soft_clip = in)); }

void Load::callback(bool)
{
  if (! _plugin) return;
  
  plugin_update();
  sig.emit(_plugin);
}

void Load::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    Str tmp;
    tmp << _option.color;
    _plugin->option(_plugin->option_list()[LOAD_COLOR], &tmp);
    tmp << _option.print;
    _plugin->option(_plugin->option_list()[LOAD_PRINT], &tmp);
  }

  callbacks(true);
}

void Load::widget_update()
{
  callbacks(false);
 
try
{
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[LOAD_COLOR]);
    tmp >> _option.color;
    tmp = _plugin->option(_plugin->option_list()[LOAD_PRINT]);
    tmp >> _option.print;
  }
}
catch (Str) {}

  _p->color->set(_option.color);
  _p->black->set(_option.print.black);
  _p->white->set(_option.print.white);
  _p->gamma->set(_option.print.gamma);
  _p->soft_clip->set(_option.print.soft_clip);

  callbacks(true);
}

struct Save::_Save
{
  choice::Widget * color;
  num::Int_Slider * black, * white;
  num::Float_Slider * gamma;
  choice::Widget * type;
  choice::Widget * endian;
};

Save::Save() :
  _plugin(0),
  _p(new _Save)
{
  // Widgets.
  
  group::Box * color_group =
    new group::Box(image_io_cineon::label_color_group);
  _p->color =
    new choice::Widget(djv_image::io_cineon::label_color(),
      image_io_cineon::label_color);
  
  group::Box * print_group =
    new group::Box(image_io_cineon::label_print_group);
  _p->black =
    new num::Int_Slider(0, 1023, image_io_cineon::label_print_black);
  _p->white =
    new num::Int_Slider(0, 1023, image_io_cineon::label_print_white);
  _p->gamma =
    new num::Float_Slider(0.01, 4.0, image_io_cineon::label_print_gamma);
  
  group::Box * group =
    new group::Box(label_file_group);
  _p->type =
    new choice::Widget(djv_image::io_dpx::label_type(), label_file_type);
  _p->endian =
    new choice::Widget(djv_image::io_dpx::label_endian(), label_file_endian);
  
  // Layout.

  layout::group(List<layout::Item *>() <<
    _p->black <<
    _p->white <<
    _p->gamma);
  layout::group(List<layout::Item *>() <<
    _p->type <<
    _p->endian);
  
  Layout_V * layout, * layout_group, * layout_v;
  
  layout = new Layout_V(this);

  layout->add(color_group);
  layout_group = new Layout_V(color_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->color);
  
  layout->add(print_group);
  layout_group = new Layout_V(print_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  layout_v->add(_p->black);
  layout_v->add(_p->white);
  layout_v->add(_p->gamma);

  layout->add(group);
  layout_group = new Layout_V(group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->type);
  layout_group->add(_p->endian);
  
  layout->add_stretch();
  
  // Initialize.

  _p->black->default_value(Save_Option().print.black);
  _p->white->default_value(Save_Option().print.white);
  _p->gamma->default_value(Save_Option().print.gamma);
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->color->sig.set(this, color_callback);
  _p->black->sig.set(this, black_callback);
  _p->white->sig.set(this, white_callback);
  _p->gamma->sig.set(this, gamma_callback);
  _p->type->sig.set(this, type_callback);
  _p->endian->sig.set(this, endian_callback);
}

Save::~Save()
{}

plugin::Plugin * Save::copy() const { return new Save; }

const Str & Save::name() const { return djv_image::io_dpx::name; }

void Save::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Save::color_callback(int in)
  { callback((_option.color = djv_image::io_cineon::COLOR(in))); }
void Save::black_callback(int in)
  { callback((_option.print.black = in)); }
void Save::white_callback(int in)
  { callback((_option.print.white = in)); }
void Save::gamma_callback(double in)
  { callback((_option.print.gamma = in)); }
void Save::type_callback(int in)
  { callback((_option.type = TYPE(in))); }
void Save::endian_callback(int in)
  { callback((_option.endian = ENDIAN(in))); }

void Save::callback(bool)
{
  if (! _plugin) return;
  
  plugin_update();
  sig.emit(_plugin);
}

void Save::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    Str tmp;
    tmp << _option.color;
    _plugin->option(_plugin->option_list()[SAVE_COLOR], &tmp);
    tmp << _option.print;
    _plugin->option(_plugin->option_list()[SAVE_PRINT], &tmp);
    tmp << _option.type;
    _plugin->option(_plugin->option_list()[SAVE_TYPE], &tmp);
    tmp << _option.endian;
    _plugin->option(_plugin->option_list()[SAVE_ENDIAN], &tmp);
  }

  callbacks(true);
}

void Save::widget_update()
{
  callbacks(false);

try
{  
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[SAVE_COLOR]);
    tmp >> _option.color;
    tmp = _plugin->option(_plugin->option_list()[SAVE_PRINT]);
    tmp >> _option.print;
    tmp = _plugin->option(_plugin->option_list()[SAVE_TYPE]);
    tmp >> _option.type;
    tmp = _plugin->option(_plugin->option_list()[SAVE_ENDIAN]);
    tmp >> _option.endian;
  }
}
catch (Str) {}

  _p->color->set(_option.color);
  _p->black->set(_option.print.black);
  _p->white->set(_option.print.white);
  _p->gamma->set(_option.print.gamma);
  _p->type->set(_option.type);
  _p->endian->set(_option.endian);

  callbacks(true);
}

const Str
  label_file_group = "File",
  label_file_type = "Type:",
  label_file_endian = "Endian:";

}
}

