/*------------------------------------------------------------------------------
 lib/djv_ui/image_io_ppm.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/image_io_ppm.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/style.h>

namespace djv_ui {
namespace image_io_ppm {
using namespace djv_image::io_ppm;

struct Save::_Save
{
  choice::Widget * type;
  choice::Widget * mode;
};

Save::Save() :
  _plugin(0),
  _p(new _Save)
{
  // Widgets.
  
  group::Box * type_group =
    new group::Box(label_type_group);
  _p->type =
    new choice::Widget(djv_image::io_ppm::label_type(), label_type);

  group::Box * mode_group =
    new group::Box(label_mode_group);
  _p->mode =
    new choice::Widget(djv_image::io_ppm::label_mode(), label_mode);
  
  // Layout.

  Layout_V * layout, * layout_group;
  
  layout = new Layout_V(this);
  
  layout->add(type_group);
  layout_group = new Layout_V(type_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->type);

  layout->add(mode_group);
  layout_group = new Layout_V(mode_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->mode);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->type->sig.set(this, type_callback);
  _p->mode->sig.set(this, mode_callback);
}

Save::~Save()
{}

plugin::Plugin * Save::copy() const { return new Save; }

const Str & Save::name() const { return djv_image::io_ppm::name; }

void Save::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Save::type_callback(int in)
{
  if (! _plugin) return;
  
  _option.type = TYPE(in);
  
  plugin_update();
  sig.emit(_plugin);
}

void Save::mode_callback(int in)
{
  if (! _plugin) return;
  
  _option.mode = MODE(in);
  
  plugin_update();
  sig.emit(_plugin);
}

void Save::plugin_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    Str tmp;
    tmp << _option.type;
    _plugin->option(_plugin->option_list()[SAVE_TYPE], &tmp);
    tmp << _option.mode;
    _plugin->option(_plugin->option_list()[SAVE_MODE], &tmp);
  }
  
  callbacks(true);
}

void Save::widget_update()
{
  callbacks(false);

try
{
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[SAVE_TYPE]);
    tmp >> _option.type;
    tmp = _plugin->option(_plugin->option_list()[SAVE_MODE]);
    tmp >> _option.mode;
  }
}
catch (Str) {}

  _p->type->set(_option.type);
  _p->mode->set(_option.mode);
  
  callbacks(true);
}

const Str
  label_type_group = "Type",
  label_type = "Value:",
  label_mode_group = "Mode",
  label_mode = "Value:";

}
}

