/*------------------------------------------------------------------------------
 lib/djv_ui/style.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/app.h>
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Tooltip.H>

namespace djv_ui {
namespace style {

namespace {

const djv_image::Color _color_foreground [] =
{
  djv_image::Color(1.0),
  djv_image::Color(1.0),
  djv_image::Color(0.0)
};

const djv_image::Color _color_background [] =
{
  djv_image::Color(0.4),
  djv_image::Color(0.6),
  djv_image::Color(0.8)
};

const djv_image::Color _color_background2 [] =
{
  djv_image::Color(0.3),
  djv_image::Color(0.5),
  djv_image::Color(0.7)
};

const djv_image::Color _color_select [] =
{
  djv_image::Color(1.0, 0.9, 0.5),
  djv_image::Color(1.0, 0.9, 0.5),
  djv_image::Color(1.0, 0.9, 0.5)
};

const int _size_pick [] = { 10, 10, 10 };
const int _size_handle [] = { 20, 40, 60 };

const int _font_size [] = { 10, 12, 14 };

const int _spacing [] = { 4, 4, 4 };
const int _spacing_large [] = { 16, 16, 16 };

const int _margin [] = { 4, 4, 4 };
const int _margin_text [] = { 2, 2, 2 };
const int _margin_widget [] = { 4, 4, 4 };
const int _margin_large [] = { 16, 16, 16 };

const int _size_icon = 22;
const int _size_tool_icon = 11;
const int _size_button [] = { 40, 50, 60 };
const int _size_text [] = { 100, 120, 140 };
const int _size_swatch [] = { 40, 50, 60 };
const int _size_thumbnail [] = { 60, 80, 100 };

}

Color::Color() :
  foreground(_color_foreground[COLOR_NEUTRAL]),
  background(_color_background[COLOR_NEUTRAL]),
  background2(_color_background2[COLOR_NEUTRAL]),
  select(_color_select[COLOR_NEUTRAL])
{}

class Style::_Style
{
public:

  _Style();
  
  void color(COLOR);
  void color_custom(const Color &);
  COLOR color() const { return _color; }
  const Color & color_value() const { return _color_value; }
  const Color & color_custom() const { return _color_custom; }

  void size(SIZE);
  SIZE size() const { return _size; }

private:

  void color_update();
  void size_update();
  
  COLOR _color;
  Color _color_value;
  Color _color_custom;
  SIZE _size;
};

Style::_Style::_Style() :
  _color(prefs()->color()),
  _color_custom(prefs()->color_custom()),
  _size(prefs()->size())
{
  color_update();
  size_update();
}

void Style::_Style::color(COLOR in)
{
  if (in == _color) return;

  _color = in;

  color_update();
}

void Style::_Style::color_custom(const Color & in)
{
  if (in == _color_custom) return;

  _color_custom = in;

  color_update();
}

void Style::_Style::color_update()
{
  DJV_DBG_OBJ("Style::_Style::color_update");
  
  switch (_color)
  {
    case COLOR_CUSTOM:
      _color_value = _color_custom;
      break;

    default:
      _color_value.foreground = _color_foreground[_color];
      _color_value.background = _color_background[_color];
      _color_value.background2 = _color_background2[_color];
      _color_value.select = _color_select[_color];
      break;
  }

  djv_image::Color tmp(djv_image::pixel::RGB_U8);
  djv_image::color::convert(_color_value.background, &tmp);
  Fl::background(tmp.get_u8(0), tmp.get_u8(1), tmp.get_u8(2));
  djv_image::color::convert(_color_value.background2, &tmp);
  Fl::background2(tmp.get_u8(0), tmp.get_u8(1), tmp.get_u8(2));
  djv_image::color::convert(_color_value.foreground, &tmp);
  Fl::foreground(tmp.get_u8(0), tmp.get_u8(1), tmp.get_u8(2));
  djv_image::color::convert(_color_value.select, &tmp);
  Fl::set_color(FL_SELECTION_COLOR, tmp.get_u8(0), tmp.get_u8(1),
    tmp.get_u8(2));
  
  app::global()->window_redraw();
}

void Style::_Style::size(SIZE in)
{
  if (in == _size) return;
  
  _size = in;
  
  size_update();
}

void Style::_Style::size_update()
{
  DJV_DBG_OBJ("Style::_Style::size_update");

  Fl_Tooltip::size(_font_size[_size]);
  
  app::global()->window_update();
}

Style::Style() :
  _p(new _Style)
{}

Style::~Style()
{}

void Style::color(COLOR in) { _p->color(in); }
COLOR Style::color() const { return _p->color(); }
const Color & Style::color_value() const { return _p->color_value(); }

void Style::color_custom(const Color & in) { _p->color_custom(in); }
const Color & Style::color_custom() const { return _p->color_custom(); }

void Style::size(SIZE in) { _p->size(in); }
SIZE Style::size() const { return _p->size(); }

int Style::size_pick() const { return _size_pick[_p->size()]; }
int Style::size_handle() const { return _size_handle[_p->size()]; }

Fl_Font Style::font() const { return FL_HELVETICA; }
Fl_Font Style::font_bold() const { return FL_HELVETICA_BOLD; }
int Style::font_size() const { return _font_size[_p->size()]; }

int Style::spacing() const { return _spacing[_p->size()]; }
int Style::spacing_large() const { return _spacing_large[_p->size()]; }
int Style::margin() const { return _margin[_p->size()]; }
int Style::margin_text() const { return _margin_text[_p->size()]; }
int Style::margin_widget() const { return _margin_widget[_p->size()]; }
int Style::margin_large() const { return _margin_large[_p->size()]; }

int Style::size_icon() const { return _size_icon; }
int Style::size_tool_icon() const { return _size_tool_icon; }
int Style::size_button() const { return _size_button[_p->size()]; }
int Style::size_text() const { return _size_text[_p->size()]; }
int Style::size_swatch() const { return _size_swatch[_p->size()]; }
int Style::size_thumbnail() const { return _size_thumbnail[_p->size()]; }

}
}

