/*------------------------------------------------------------------------------
 lib/djv_ui/tab.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/layout_stack.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/style.h>
#include <djv_ui/scroll.h>
#include <djv_ui/tab.h>
#include <FL/Fl.H>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace tab {

namespace {

class _Widget : public widget::Widget
{
public:

  _Widget(widget::Base *);
  
  void name(const Str &);

  const Str & name() const { return _name; }
  
  Str dbg_name() const { return "djv::ui::tab::_Widget"; }
  
private:

  Str _name;
  mem::Buffer<char> _name_cstr;
};

_Widget::_Widget(widget::Base * item)
{
  text_font(style::global()->font_bold());
  
  //Layout_V * layout = new Layout_V(this);
  //if (item) layout->add(item);
  
  scroll::Widget * scroll_widget = new scroll::Widget;
  Layout_V * layout = new Layout_V(this);
  layout->add(scroll_widget);
  scroll_widget->layout()->margin(0);
  if (item) scroll_widget->layout()->add(item);
}

void _Widget::name(const Str & in)
{
  if (in == _name) return;
  
  _name = in;
  
  const int size = _name.size() + 1;
  _name_cstr.size(size);
  str::cstr(_name, _name_cstr(), size);
  Fl_Group::label(_name_cstr());
}

}

Widget::Widget() :
  Fl_Tabs(0, 0, 0, 0),
  sig(this)
{
  Fl_Tabs::end();

  layout::Stack * layout = new layout::Stack(this);
  layout->margin(0);
  
  callback(widget_callback, this);
}

Widget::~Widget() {}

void Widget::set(int in)
{
  DJV_ASSERT(in < int(_list.size()));
  
  value(dynamic_cast<Fl_Widget *>(_list[in]));
}

int Widget::get() const
{
  // XXX FLTK forgot their const.

  Widget * widget = const_cast<Widget *>(this);
  
  for (uint i = 0; i < _list.size(); ++i)
    if (widget->value() == dynamic_cast<Fl_Widget *>(_list[i]))
      return i;
      
  return -1;
}

void Widget::tab_add(const Str & name, widget::Base * in)
{
  tab_insert(name, in, int(_list.size()));
}

void Widget::tab_insert(const Str & name, widget::Base * in, int index)
{
  DJV_DBG_OBJ("Widget::tab_insert");
  DJV_DBG_PRN("name = " << name);
  DJV_DBG_PRN("index = " << index);
  
  _Widget * widget = new _Widget(in);
  widget->name(name);
  
  List<widget::Base *>::iterator i = _list.begin();
  int j = 0;
  for (; j < index && j < int(_list.size()); ++j, ++i) ;
  _list.insert(i, widget);
  
  layout()->insert(widget, j);
  
  //dirty();
  update();
  redraw();
  
  //tab(index);
}

void Widget::tab_name(const Str & name, int tab)
{
  DJV_ASSERT(tab < int(_list.size()));
  
  DJV_DBG_OBJ("Widget::tab_name");
  DJV_DBG_PRN("name = " << name);
  DJV_DBG_PRN("tab = " << tab);
  
  static_cast<_Widget *>(_list[tab])->name(name);
  
  //dirty();
  update();
  redraw();
}

void Widget::tab_del(int in)
{
  DJV_ASSERT(in < int(_list.size()));
  
  DJV_DBG_OBJ("Widget::tab_del");
  DJV_DBG_PRN("in = " << in);

  List<widget::Base *>::iterator i = _list.begin();
  for (int j = 0; j < in; ++j, ++i) ;
  _list.erase(i);
  
  layout()->del(in);
  
  //dirty();
  update();
  redraw();
}

void Widget::tab_del()
{
  while (_list.size()) tab_del(0);
}
  
int Widget::tab_size() const { return _list.size(); }

int Widget::handle(int in)
{
  switch (widget::Base::handle(in))
  {
    case -1: return 0;
    case 1: return 1;
  }
  return Fl_Tabs::handle(in);
}
  
void Widget::show()
{
  Fl_Tabs::show();
  widget::Base::show();
}

void Widget::draw()
{
  Fl_Tabs::draw();

  //const Box2i & geom = this->geom();

  /*// XXX Erase extra tab lines.
  
  fl_color(FL_BACKGROUND_COLOR);
  fl_line(geom.x, geom.y + _size_tab.y, geom.x, geom.y + geom.h - 1);
  fl_line(geom.x + geom.w - 1, geom.y + _size_tab.y, geom.x + geom.w - 1,
    geom.y + geom.h - 1);
  fl_line(geom.x, geom.y + geom.h - 1, geom.x + geom.w - 1,
    geom.y + geom.h - 1);*/
  
  widget::Base::draw();
}

void Widget::dirty()
{
  DJV_DBG_OBJ("Widget::dirty");
  
  widget::Base::dirty();

  fl_font(text_font(), style::global()->font_size());
  int label_hint = 0;
  for (uint i = 0; i < _list.size(); ++i)
    label_hint = math::max(
      base::str_width(static_cast<_Widget *>(_list[i])->name()),
      label_hint);

  _size_tab =
    Vec2i(0, fl_height()) +
    style::global()->margin_widget() * 2 +
    Vec2i(1 * 2, 1);
  DJV_DBG_PRN("tab = " << _size_tab);

  size_hint(_size_tab + layout()->size_hint() + 1 * 2);
}

void Widget::geom(const Box2i & in)
{
  DJV_DBG_OBJ("Widget::geom");
  DJV_DBG_PRN("in = " << in);
  
  Fl_Widget::resize(in.x, in.y, in.w, in.h);
  layout::Item::geom(in);
  layout()->geom(Box2i(
    in.x + 1,
    in.y + _size_tab.y + 1,
    in.w - 1 * 2,
    in.h - _size_tab.y - 1 * 2));
}

void Widget::resize(int x, int y, int w, int h)
{
  geom(Box2i(x, y, w, h));
}

const Box2i & Widget::geom() const
{
  return widget::Base::geom();
}

int Widget::visible() const
{
  return Fl_Tabs::visible();
}

void Widget::widget_callback()
{
  sig.emit(get());
}

}
}

