/*------------------------------------------------------------------------------
 plugin/djv_exr_base/etc.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "base.h"

namespace djv_exr {

Box2i imf_box(const Imath::Box2i & in)
{
  return Box2i(
    Vec2i(in.min.x, in.min.y),
    Vec2i(in.max.x, in.max.y) - Vec2i(in.min.x, in.min.y) + 1);
}

Imf::PixelType imf_type(pixel::TYPE in)
{
  switch (in)
  {
    case pixel::U8:
    case pixel::U10:
    case pixel::U16: 
    case pixel::F16: return Imf::HALF;
    default: break;
  }
  return Imf::FLOAT;
}

pixel::TYPE imf_type(Imf::PixelType in)
{
  switch (in)
  {
    case Imf::HALF: return pixel::F16;
    default: break;
  }
  return pixel::F32;
}

DJV_STR_OPERATOR_SET(COLOR, label_color())
DJV_STR_OPERATOR_GET(COLOR, label_color())
DJV_STR_OPERATOR_SET(COMPRESS, label_compress())
DJV_STR_OPERATOR_GET(COMPRESS, label_compress())

const List<Str> & label_color()
{
  static const List<Str> data = List<Str>() <<
    "Raw" <<
    "Exposure";
  return data;
}

const List<Str> & label_compress()
{
  static const List<Str> data = List<Str>() <<
    "None" <<
    "RLE" <<
    "ZIPS" <<
    "ZIP" <<
    "PIZ" <<
    "PXR24" <<
    "B44" <<
    "B44A";
  return data;
}

const List<Str> & label_tag()
{
  static const List<Str> data = List<Str>() <<
    "Longitude" <<
    "Latitude" <<
    "Altitude" <<
    "Focus" <<
    "Exposure" <<
    "Aperture" <<
    "ISO Speed" <<
    "Chromaticities" <<
    "White Luminance" <<
    "X Density";
  return data;
}

}

