/*------------------------------------------------------------------------------
 plugin/djv_exr/io.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_EXR_IO_H
#define DJV_EXR_IO_H

#include "base.h"
#include <djv_image/io.h>

namespace djv_exr {
namespace io {

//------------------------------------------------------------------------------
// Options

enum LOAD_OPTION
{
  LOAD_COLOR,
  LOAD_EXPOSURE
};

struct Load_Option
{
  Load_Option() :
    color(COLOR_EXPOSURE)
  {}

  COLOR color;
  profile::Exposure exposure;
};
  
enum SAVE_OPTION
{
  SAVE_COMPRESS
};

struct Save_Option
{
  Save_Option() :
    compress(COMPRESS_NONE)
  {}

  COMPRESS compress;
};

//------------------------------------------------------------------------------
// Load Plugin

class Load : public djv_image::io::Load
{
public:

  Load();
  ~Load();
  
  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, Info *, Tag *, djv_image::Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, djv_image::Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:
  
  void _open(const File &, List<Info> *, Tag *, djv_image::Time * = 0)
    throw (Error);
  void _close();
  
  Load_Option _option;
  File _file;
  Imf::InputFile * _f;
  List<List<Channel> > _layer_list;
  Image _image;
  Data _tmp;
  Cache _cache;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public djv_image::io::Save
{
public:

  Save();
  ~Save();
  
  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, const Info &, const Tag &, const djv_image::Time &)
    throw (Error);
  void set(const Image &, int frame) throw (Error);

private:

  void _open(const File &, const Tag &) throw (Error);
  void _close();

  Save_Option _option;
  File _file;
  Imf::OutputFile * _f;
  List<Str> _channel_list;
  Info _info;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_load_option();
const List<Str> & label_save_option();

}
}

#endif

