/*------------------------------------------------------------------------------
 plugin/djv_quicktime/base.h
 
 Image I/O - QuickTime
 ---------------------
 
 Apple QuickTime movie file format.
 
 Requires:
 
 * QuickTime - http://www.apple.com/quicktime

 Support:
 
 * Image data: RGBA, 8-bit.
 * File compression.
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_QUICKTIME_BASE_H
#define DJV_QUICKTIME_BASE_H

#include <djv_image/base.h>
namespace qt {
#if defined(DJV_MACOSX)
#include <QuickTime/QuickTime.h>
#elif defined(WIN32)
#include <Movies.h>
#endif
#define qt_fixed1 ((qt::Fixed)0x00010000L) // XXX
}

namespace djv_quicktime {
using namespace djv_base;
using namespace djv_image;

//------------------------------------------------------------------------------
// Information

static const Str name = "QuickTime";

static const List<Str> extension_list = List<Str>() <<
  ".qt" <<
  ".mov" <<
  ".avi" <<
  ".mp4";

//------------------------------------------------------------------------------
// Options

enum CODEC
{
  CODEC_RAW,
  CODEC_JPEG,
  CODEC_MJPEGA,
  CODEC_MJPEGB,
  CODEC_H263,
  CODEC_H264,
  CODEC_DVCNTSC,
  CODEC_DVCPAL
};

enum QUALITY
{
  QUALITY_LOSSLESS,
  QUALITY_MIN,
  QUALITY_MAX,
  QUALITY_LOW,
  QUALITY_NORMAL,
  QUALITY_HIGH
};

//------------------------------------------------------------------------------
// Utility

class Handle
{
public:

  Handle(qt::Handle = 0);
  ~Handle();

  void init(long) throw (Error);

  qt::Handle p;
};

class CFStringRef
{
public:
  
  CFStringRef(const Str &);
  ~CFStringRef();

  qt::CFStringRef p;
};

//------------------------------------------------------------------------------
// Functions

void quicktime_init() throw (Error);
void quicktime_del();

qt::CodecType quicktime_codec(CODEC);
qt::CodecQ quicktime_quality(QUALITY);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, CODEC &) throw (Str);
Str & operator >> (Str &, QUALITY &) throw (Str);
Str & operator << (Str &, CODEC);
Str & operator << (Str &, QUALITY);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_codec();
const List<Str> & label_quality();

extern const Str
  error_init,
  error_version,
  error_open,
  error_gworld,
  error_track,
  error_media,
  error_media_begin,
  error_media_end,
  error_media_add,
  error_media_sample,
  error_movie_add,
  error_compress,
  error_handle;

}

#endif

