/*------------------------------------------------------------------------------
 plugin/djv_quicktime/io.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_QUICKTIME_IO_H
#define DJV_QUICKTIME_IO_H

#include "base.h"
#include <djv_image/io.h>

namespace djv_quicktime {
namespace io {
using namespace djv_base;
using namespace djv_image;

//------------------------------------------------------------------------------
// Options

enum SAVE_OPTION
{
  SAVE_CODEC,
  SAVE_QUALITY
};

struct Save_Option
{
  Save_Option() :
    codec(CODEC_JPEG),
    quality(QUALITY_NORMAL)
  {}

  CODEC codec;
  QUALITY quality;
};

//------------------------------------------------------------------------------
// Load Plugin

class Load : public djv_image::io::Load
{
public:

  Load();
  ~Load();

  void init() throw (Error);
  void del();
  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, Info *, Tag *, djv_image::Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, djv_image::Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _close();
  
  File _file;
  Info _info;
  djv_image::Time _time;
  qt::Movie _movie;
  qt::GWorldPtr _gworld;
  Image _image, _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public djv_image::io::Save
{
public:

  Save();
  
  void init() throw (Error);
  void del();
  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, const Info &, const Tag &, const djv_image::Time &)
    throw (Error);
  void close() throw (Error);
  void set(const Image &, int frame) throw (Error);

private:

  void _open(const File &, const Info &) throw (Error);
  
  Save_Option _option;
  File _file;
  int _frame_duration;
  //short _f;
  qt::DataHandler _f;
  qt::Movie _movie;
  qt::Track _track;
  qt::Media _media;
  qt::GWorldPtr _gworld;
  Info _info;
  Image _image;
};

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_save_option();

}
}

#endif

