/*------------------------------------------------------------------------------
 plugin/djv_quicktime_unix/io.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_QUICKTIME_UNIX_IO_H
#define DJV_QUICKTIME_UNIX_IO_H

#include "base.h"
#include <djv_image/io.h>

namespace djv_quicktime {
namespace io {

//------------------------------------------------------------------------------
// Options

enum SAVE_OPTION
{
  SAVE_CODEC
};

struct Save_Option
{
  Save_Option() :
    codec("jpeg")
  {}

  Str codec;
};

//------------------------------------------------------------------------------
// Load Plugin

class Load : public djv_image::io::Load
{
public:

  Load();
  ~Load();

  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, Info *, Tag *, djv_image::Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, djv_image::Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _close();
  
  File _file;
  Info _info;
  djv_image::Time _time;
  int _frame;
  quicktime_t * _f;
  Image _image;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public djv_image::io::Save
{
public:

  Save();
  
  plugin::Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, const Info &, const Tag &, const djv_image::Time &)
    throw (Error);
  void close() throw (Error);
  void set(const Image &, int frame) throw (Error);

private:
  
  Save_Option _option;
  File _file;
  quicktime_t * _f;
  int _frame, _frame_duration;
  Info _info;
  Image _image;
};

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_save_option();

}
}

#endif

