/*------------------------------------------------------------------------------
 plugin/djv_vlut/base.h
 
 Image I/O - VLUT
 ----------------
 
 Video lookup table I/O.
 
 Video lookup tables control the colors shown on a display device.

 Requires:

 * Linux - X11 Video Mode Extension, http:://www.x.org
 * SGI IRIX - X11 Video Control Extension, http:://www.sgi.org/irix
 
 Support:

 * Image data: RGB, 8-bit, 16-bit (platform-dependent).

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VLUT_BASE_H
#define DJV_VLUT_BASE_H

#include <djv_image/base.h>
#if defined(WIN32)
#include <windows.h>
#elif defined(DJV_IRIX)
#include <X11/Xlib.h>
#include <X11/extensions/XSGIvc.h>
#else
#include <X11/Xlib.h>
#include <X11/extensions/xf86vmode.h>
#endif

namespace djv_vlut {
using namespace djv_base;
using namespace djv_image;

//------------------------------------------------------------------------------
// Information

static const Str name = "VLUT";

static const List<Str> extension_list = List<Str>() << ".vlut";

//------------------------------------------------------------------------------
// Video LUT Handle

#if defined(WIN32)

struct Handle
{
  Handle();
  ~Handle();

  void open() throw (Error);
  void close();
  
  HDC operator () () const;

private:

  HDC _display;
};

#endif

}

#endif

