#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/tab.h>
#include <djv_ui/image_io.h>
#include <djv_ui/app.h>

class App : public djv_ui::app::App
{
public:

  App(int & argc, char ** argv);
  ~App();

private:

  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  djv_ui::window::Widget * _window;
};

App::App(int & argc, char ** argv) :
  djv_ui::app::App("image_io", argc, argv),
  _window(0)
{
  using namespace djv_ui;

  valid(true);
  
  // Widgets.
  
  tab::Widget * tab_widget = new tab::Widget;

  tab::Widget * load_tab_widget = new tab::Widget;
  tab_widget->tab_add("Load", load_tab_widget);
  const List<djv_image::io::Load *> & load_list =
    djv_image::io::global()->load()->list();
  for (uint i = 0; i < load_list.size(); ++i) {
    image_io::Load * widget = image_io::global()->load()->get(load_list[i]);
    if (! widget) continue;
    widget->set(load_list[i]);
    load_tab_widget->tab_add(load_list[i]->name(), widget);
  }

  tab::Widget * save_tab_widget = new tab::Widget;
  tab_widget->tab_add("Save", save_tab_widget);
  const List<djv_image::io::Save *> & save_list =
    djv_image::io::global()->save()->list();
  for (uint i = 0; i < save_list.size(); ++i) {
    image_io::Save * widget = image_io::global()->save()->get(save_list[i]);
    if (! widget) continue;
    widget->set(save_list[i]);
    save_tab_widget->tab_add(save_list[i]->name(), widget);
  }

  _window = new djv_ui::window::Widget(name());

  // Layout.
  
  Layout_V * layout = new Layout_V(_window);
  layout->margin(0);
  layout->add(tab_widget);
  
  // Initialize.
  
  _window->size(Vec2i(500, 400));
  _window->show();

  // Callbacks.
  
  _window->close_sig.set(this, exit_callback);
}

App::~App()
{
  if (_window) _window->del();
}

int main(int argc, char ** argv) { return App(argc, argv).exec(); }

