//#define DJV_DBG

#include "app.h"
#include "op_color.h"
#include "op_levels.h"
#include "op_exposure.h"
#include "op_blur.h"
#include "op_sharpen.h"
#include "op_edge.h"
#include "op_scale.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/tab.h>
#include <FL/Fl.H>

namespace glsl {

App::App(int argc, char ** argv) :
  djv_ui::app::App("glsl", argc, argv),
  _play(false),
  _frame(0),
  _op(0),
  _window_image(0),
  _window_widget(0)
{
  DJV_DBG_OBJ("App::App");
  
  djv_image::gl_offscreen::global()->buffer(djv_image::gl_offscreen::PBUFFER);

  if (argc != 2)
  {
    print("usage: test_glsl (image)");
    return;
  }
  
  try
  {
    File file(argv[1]);
    if (file.seq_valid())
      file.type = djv_base::file::SEQ;
    
    _load = std::auto_ptr<djv_image::io::Load>(
      djv_image::io::global()->load(file, &_info, &_tag, &_time));
  }
  catch (Error error)
  {
    this->error(error);
    return;
  }
  
  valid(true);
  
  _op_list += new Op_Null();
  _op_list += new op_color::Op();
  _op_list += new op_levels::Op();
  _op_list += new op_exposure::Op();
  _op_list += new op_blur::Op();
  _op_list += new op_sharpen::Op();
  _op_list += new op_edge::Op();
  _op_list += new op_scale::Op();
  
  _window_image = new djv_ui::window::Widget;
  _view = new Widget;
  _play_widget = new djv_ui::button::Tool("playback_forward");
  _play_widget->type(djv_ui::button::Tool::TOGGLE);
  _slider = new djv_ui::slider::Int;
  
  _window_widget = new djv_ui::window::Widget;
  djv_ui::tab::Widget * tab_widget = new djv_ui::tab::Widget;
  for (uint i = 0; i < _op_list.size(); ++i)
    tab_widget->tab_add(_op_list[i]->name(), _op_list[i]);
  
  djv_ui::Layout_V * layout = new djv_ui::Layout_V(_window_image);
  layout->margin(0);
  layout->spacing(0);
  layout->add(_view);
  layout->stretch(_view);
  djv_ui::Layout_H * layout_h = new djv_ui::Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(_play_widget);
  layout_h->add(_slider);
  
  layout = new djv_ui::Layout_V(_window_widget);
  layout->margin(0);
  layout->spacing(0);
  layout->add(tab_widget);

  update();
  
  _slider->range(0, int(_time.list.size()) - 1);
  _window_widget->size(300, 300);
  _window_widget->show();
  _window_image->dirty();
  _window_image->size(_info.size + Vec2i(0, _window_image->size_min().y));
  _window_image->show();
  tab_widget->set(_op);
  
  _window_image->close_sig.set(this, close_callback);
  _play_widget->sig.set(this, play_callback);
  _slider->sig.set(this, frame_callback);
  tab_widget->sig.set(this, op_callback);
  for (uint i = 0; i < _op_list.size(); ++i)
    _op_list[i]->sig.set(this, update_callback);
  _window_widget->close_sig.set(this, close_callback);
}

App::~App()
{
  DJV_DBG_OBJ("App::~App");
  
  if (_window_image) _window_image->del();
  if (_window_widget) _window_widget->del();
}

void App::play_callback(bool in)
{
  _play = in;
  play_update();
}

void App::frame_callback(int in)
{
  _play = false;
  _frame = in;
  play_update();
  frame_update();
  update();
}

void App::op_callback(int in)
{
  _op = in;
  update();
}

void App::update_callback(bool)
{
  update();
}

void App::play_update()
{
  callbacks(false);
  
  if (_play)
    Fl::add_idle(idle_callback, this);
  else
    Fl::remove_idle(idle_callback, this);
  
  _play_widget->set(_play);
  
  callbacks(true);
}
  
void App::frame_update()
{
  callbacks(false);
  
  _slider->set(_frame);
  
  callbacks(true);
}

void App::update()
{
  callbacks(false);
  
  try
  {
    const int frame = _time.list.size() ? _time.list[_frame] : -1;
    _view->set(_op_list[_op], _load->get(frame));
    _view->redraw();
  }
  catch (Error)
  {
  }
  
  callbacks(true);
}

void App::close_callback(bool)
{
  exit();
}

void App::idle_callback()
{
  _frame++;
  if (_frame >= int(_time.list.size())) _frame = 0;
  
  frame_update();
  update();
}

}

int main(int argc, char ** argv) { return glsl::App(argc, argv).exec(); }
